import logging
import re
import typing

import conscommon.data_model
from pydm.widgets import channel
from qtpy.QtCore import QObject, Signal

logger = logging.getLogger()


class TableDataRow:
    def __init__(
        self,
        device: conscommon.data_model.Device,
        channel: conscommon.data_model.Channel,
        render: bool,
    ) -> None:
        self.device: conscommon.data_model.Device = device
        self.channel: conscommon.data_model.Channel = channel
        self.render: bool = render


class TableDataController(QObject):

    update_content = Signal()

    _table_batch = 24

    def __init__(
        self,
        table,
        devices: typing.List[conscommon.data_model.Device] = None,
        table_batch=24,
        horizontal_header_labels=None,
        **kwargs
    ):
        super().__init__()
        self.devices: typing.List[conscommon.data_model.Device] = (
            devices if devices else []
        )
        self._filter_pattern: typing.Optional[str] = None
        self._table_batch = table_batch

        self.horizontalHeaderLabels = horizontal_header_labels
        self.table = table
        self.table_data: typing.List[TableDataRow] = []

        self.MaxValue = 50
        self.MinValue = 1

        self.batch_offset: int = 0
        self.total_rows: int = 0

        self.init_table()
        self.update_content.connect(self.update_table_content)
        self.update_content.connect(self.resize)

        self.load_table_data()

    def resize(self):
        if self.table:
            self.table.resizeColumnsToContents()

    def init_table(self):
        self.table.setRowCount(self._table_batch)
        self.table.setColumnCount(len(self.horizontalHeaderLabels))
        self.table.setHorizontalHeaderLabels(self.horizontalHeaderLabels)

        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def filter(self, pattern):
        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def load_table_data(self):
        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def update_table_content(self):
        # TODO: Impplement ...
        raise NotImplementedError("Subclass must implement abstract method")

    def disconnect_widget(self, row, col):
        widget = self.table.cellWidget(row, col)
        widget.channel = None

    def connect_widget(
        self, row, col, channel_name=None, macros=None, connect_color=None
    ):
        widget = self.table.cellWidget(row, col)
        if channel_name:
            widget.channel = channel_name
        if macros:
            widget.macros = macros
        if connect_color:
            cell_color = channel.PyDMChannel(
                address="ca://" + channel_name, value_slot=lambda: self.setColor(row)
            )
            cell_color.connect()

    def setColor(
        self, row, col=3
    ):  # Function to define cell color according to the value
        try:
            vmax = self.MaxValue
            vmin = self.MinValue
            range = vmax - vmin
            cell = self.table.cellWidget(row, col)

            if cell.text() != "":
                ActualValue = float(re.match(r"(\d+)\.(\d+)", cell.text()).group())

                if ActualValue < vmin:
                    ActualValue = vmin
                if (ActualValue - vmin) / range <= 0.5:
                    R = int((510 / range) * (ActualValue - vmin))
                    G = 255

                elif ActualValue < 200:
                    if ActualValue > vmax:
                        ActualValue = vmax
                    R = 255
                    G = int(-(510 / range) * (ActualValue - vmax))

                else:
                    return ()
                cell.setStyleSheet("background-color: rgb{}".format((R, G, 00)))
        except Exception:
            print("Problema com setColor")
            logger.exception('Problema com "setColor"')

    def changeBatch(self, increase):
        if increase:
            if self.batch_offset < len(self.table_data):
                self.batch_offset += self._table_batch
                self.update_content.emit()
        else:
            if self.batch_offset != 0:
                self.batch_offset -= self._table_batch
                if self.batch_offset < 0:
                    self.batch_offset = 0
                self.update_content.emit()



import pkg_resources
from qtpy.QtGui import QIcon, QPixmap


def get_abs_path(relative):
    return pkg_resources.resource_filename(__name__, relative)


_CNPEM_IMG: str = get_abs_path("resources/images/CNPEM.jpg")
_CNPEM_INVISIBLE_IMG: str = get_abs_path("resources/images/CNPEM-invisible.png")
_CNPEM_INVISIBLE_LOGO_IMG: str = get_abs_path(
    "resources/images/CNPEM-invisible-logo.png"
)
_LNLS_IMG: str = get_abs_path("resources/images/sirius-hla-as-cons-lnls.png")
_LNLS_INVISIBLE_IMG: str = get_abs_path("resources/images/lnls-invisible.png")

CNPEM_PIXMAP: QPixmap = QPixmap(_CNPEM_IMG)
CNPEM_ICON: QIcon = QIcon(CNPEM_PIXMAP)

CNPEM_INVISIBLE_PIXMAP: QPixmap = QPixmap(_CNPEM_INVISIBLE_IMG)
CNPEM_INVISIBLE_ICON: QIcon = QIcon(CNPEM_INVISIBLE_PIXMAP)

CNPEM_INVISIBLE_LOGO_PIXMAP: QPixmap = QPixmap(_CNPEM_INVISIBLE_LOGO_IMG)
CNPEM_INVISIBLE_LOGO_ICON: QIcon = QIcon(CNPEM_INVISIBLE_LOGO_PIXMAP)

LNLS_PIXMAP: QPixmap = QPixmap(_LNLS_IMG)
LNLS_ICON: QIcon = QIcon(LNLS_PIXMAP)

LNLS_INVISIBLE_PIXMAP: QPixmap = QPixmap(_LNLS_INVISIBLE_IMG)
LNLS_INVISIBLE_ICON: QIcon = QIcon(LNLS_INVISIBLE_PIXMAP)



#!/usr/local/env python3
import json
import logging

from pydm import Display
from pydm.widgets.related_display_button import PyDMRelatedDisplayButton
from qtpy.QtWidgets import QVBoxLayout, QWidget

from siriushlacon.pctrl.consts import IOCS_JSON, PCTRL_UI


def load_iocs():
    with open(IOCS_JSON, "rb") as f:
        iocs = json.load(f)
    return iocs


logger = logging.getLogger()


class Main(Display):
    def __init__(self, parent=None):
        super().__init__(parent=parent, ui_filename=PCTRL_UI)
        self.iocs = load_iocs()
        self.areas = set([e["area"] for e in self.iocs])
        self.tabs = {}
        self.init_layout()

    def init_layout(self):
        for area in self.areas:
            self.tabs[area] = QWidget()
            self.tabs[area].setLayout(QVBoxLayout())
            self.tabWidget.addTab(self.tabs[area], area)

        for ioc in self.iocs:
            button = PyDMRelatedDisplayButton(ioc["desc"])
            button.filenames = ["ui/procServControl.ui"]
            button.macros = '{"P":"' + ioc["pv"] + '"}'
            button.showIcon = False
            button.openInNewWindow = True
            self.tabs[ioc["area"]].layout().addWidget(button)

        for area in self.areas:
            self.tabs[area].layout().addStretch(0)



#!/usr/bin/env python3
import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


GAMMA_COUNTING_MAIN = get_abs_path("main.py")
GAMMA_COUNTING_MAIN_UI = get_abs_path("ui/main.ui")

OVERVIEW_UI = get_abs_path("ui/overview.ui")
LAYOUT_OVERVIEW_UI = get_abs_path("ui/layout_overview.ui")

BEFORE_BC_IMAGE = get_abs_path("pic/BeforeBC.png")
AFTER_BC_IMAGE = get_abs_path("pic/AfterBC.png")
LNLS_IMAGE = get_abs_path("pic/lnlsLogo.png")
CNPEM_IMAGE = get_abs_path("pic/CNPEM.png")



import typing as _typing

import conscommon.data
import conscommon.data_model
from qtpy.QtWidgets import QApplication as _QApplication
from qtpy.QtWidgets import QMessageBox as _QMessageBox

from siriushlacon.logging import get_logger

_logger = get_logger(__name__)


class LazyDevices:
    """Helper class for lazy loading"""

    def __init__(self, api_get_method):
        self.api_get_method = api_get_method
        self._data: _typing.List[conscommon.data_model.Device] = None

    def get(self) -> _typing.List[conscommon.data_model.Device]:
        try:
            if self._data:
                return self._data

            self._data = conscommon.data_model.getDevicesFromBeagles(
                conscommon.data_model.getBeaglesFromList(self.api_get_method())
            )
            return self._data
        except Exception as e:
            _logger.exception("Failed to acquire data from the API")
            _QMessageBox.critical(
                None,
                "Beaglebone Info API Error",
                f"Failed to get information from the remote API. Check if the server is reachable at {conscommon.data.API_CANDIDATES}.\nError '{e}'",
            )
            raise e


def close_qt_application(*args, **kwargs):
    app = _QApplication.instance()
    if app:
        _logger.info(f"Closing qt application '{app}'")
        app.quit()



import logging

from siriushlacon.vbc.epics import ProcessOff, ProcessOn, ProcessRecovery

_logger = logging.getLogger(__name__)


def clear_status_on(prefix: str):
    """
    - clear all status PVs for 'Off Process'
    - clear all status PVs for 'Recovery Process'
    """
    _logger.info("clear_status_on")

    process_off = ProcessOff(prefix=prefix)
    process_recovery = ProcessRecovery(prefix=prefix)

    process_off.clear_all_fv_status()
    process_recovery.set_all_clear()


def clear_status_off(prefix: str):
    """
    - clear all status PVs for 'On Process'
    - clear all status PVs for 'Recovery Process'
    """
    _logger.info("clear_status_off")
    process_on = ProcessOn(prefix=prefix)
    process_recovery = ProcessRecovery(prefix=prefix)

    process_on.clear_all_status()
    process_recovery.set_all_clear()


def clear_status_rec(prefix: str):
    """
    - clear all status PVs for 'recovering from pressurized system' process (5*10^-2 ~ 1*10^-8)
    - clear all status PVs for 'Off Process'
    """
    process_on = ProcessOn(prefix=prefix)
    process_off = ProcessOff(prefix=prefix)

    process_on.activate_all_status()
    process_off.clear_all_fv_status()



#!/usr/bin/env python3
import logging
import re
from typing import List

import conscommon.data_model
from pydm import Display
from pydm.widgets.drawing import PyDMDrawingRectangle
from pydm.widgets.label import PyDMLabel
from qtpy.QtCore import QRect, Qt
from qtpy.QtGui import QBrush, QColor, QFont
from qtpy.QtWidgets import QFrame, QLabel

from siriushlacon.agilent4uhv.consts import lazy_devices
from siriushlacon.utils.consts import BO, OVERVIEW_UI, SI, TB, TS
from siriushlacon.widgets.layout import FlowLayout

logger = logging.getLogger()


class PVInfo:
    def __init__(
        self, PV: str, DISP: str, ALARM: str, SEC: str, RACK: str, RS485: str
    ) -> None:
        self.PV: str = PV
        self.DISP: str = DISP
        self.ALARM: str = ALARM
        self.SEC: str = SEC
        self.RACK: str = RACK
        self.RS485: str = RS485

    def __str__(self) -> str:
        return f"PVInfo({self.__dict__})"


class Overview(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(Overview, self).__init__(
            parent=parent, args=args, macros=macros, ui_filename=OVERVIEW_UI
        )

        self.ch_reg = re.compile(r":[C][0-9]")
        self._devices = lazy_devices.get()
        self.pv_info_list: List[PVInfo] = []
        for device in self._devices:
            if not device.enable:
                continue
            for channel in device.channels:
                self.load_pv_dict(device, channel)

        self.mainArea.setWidgetResizable(True)
        layout = FlowLayout(self.scrollAreaContent)
        for pv_info in self.pv_info_list:
            layout.addWidget(self.get_gauge(None, pv_info=pv_info))

    def load_pv_dict(
        self,
        device: conscommon.data_model.Device,
        channel: conscommon.data_model.Channel,
    ):
        if (
            (self.ch_reg.match(channel.prefix[-3:]))
            or (self.macros().get("TYPE") == BO and not channel.prefix.startswith(BO))
            or (self.macros().get("TYPE") == SI and not channel.prefix.startswith(SI))
            or (self.macros().get("TYPE") == TS and not channel.prefix.startswith(TS))
            or (self.macros().get("TYPE") == TB and not channel.prefix.startswith(TB))
        ):
            logger.info("Ignored {}".format(channel.prefix))
            return None

        self.pv_info_list.append(
            PVInfo(
                PV=channel.prefix + ":Pressure-Mon",
                DISP=channel.prefix + ":Pressure-Mon",
                ALARM=channel.prefix + ":Pressure-Mon.STAT",
                SEC=device.info.sector,
                RACK=device.info.rack,
                RS485=device.info.serial_id,
            )
        )

    def get_gauge(self, parent, pv_info: PVInfo):
        tooltip = "".join(
            [f"{key}\t{value}\n" for key, value in pv_info.__dict__.items()]
        )

        width = 320
        height = 100

        frame = QFrame(parent)
        frame.setGeometry(QRect(10, 10, width, height))
        frame.setMinimumSize(width, height)
        frame.setFrameShape(QFrame.StyledPanel)
        frame.setFrameShadow(QFrame.Raised)
        frame.setObjectName("frame")

        brush = QBrush(QColor(180, 180, 180))
        brush.setStyle(Qt.NoBrush)

        alarmRec = PyDMDrawingRectangle(frame)
        alarmRec.channel = "ca://{}".format(pv_info.ALARM)
        alarmRec.setGeometry(QRect(0, 0, width, height))
        alarmRec.setToolTip(tooltip)
        alarmRec.setProperty("alarmSensitiveContent", True)
        alarmRec.setProperty("brush", brush)
        alarmRec.setObjectName("alarmRec")
        # alarmRec.setStyleSheet("margin:5px; border:3px solid rgb(0, 0, 0);")

        lblName = QLabel(frame)
        lblName.setGeometry(QRect(width * 0.05, 50, width - width * 0.05, 20))
        font = QFont()
        font.setPointSize(12)
        lblName.setFont(font)
        lblName.setAlignment(Qt.AlignCenter)
        lblName.setText("{}".format(pv_info.DISP))
        lblName.setObjectName("lblName")
        lblName.setToolTip(tooltip)

        lblVal = PyDMLabel(frame)
        lblVal.setGeometry(QRect(width * 0.05, 10, width - width * 0.05, 30))
        font = QFont()
        font.setPointSize(18)
        lblVal.setFont(font)
        lblVal.setToolTip(tooltip)
        lblVal.setAlignment(Qt.AlignCenter)
        lblVal.setProperty("showUnits", False)
        lblVal.setObjectName("lblVal")
        lblVal.channel = "ca://{}".format(pv_info.PV)
        lblVal.precisionFromPV = False
        lblVal.precision = 2
        if self.macros().get("FORMAT", "") == "EXP":
            lblVal.displayFormat = PyDMLabel.DisplayFormat.Exponential
        return frame



from pydm.widgets import PyDMLabel
from qtpy.QtCore import Qt

from siriushlacon.widgets.qss import TABLE_ALARMS_QSS

_default_precision = 2


def make_cell_label(
    parent=None,
    content: str = "",
    tooltip: str = "",
    displayFormat: PyDMLabel.DisplayFormat = PyDMLabel.DisplayFormat.Default,
    precision: int = _default_precision,
    showUnits: bool = True,
    precision_from_pv: bool = False,
    alarm_sensitive_border: bool = True,
    alarm_sensitive_content: bool = True,
) -> PyDMLabel:
    if type(precision) != int or precision < 0:
        precision = _default_precision

    lbl = PyDMLabel(parent=parent, init_channel=content)
    lbl.setAlignment(Qt.AlignVCenter | Qt.AlignHCenter)
    lbl.precisionFromPV = precision_from_pv
    lbl.precision = precision
    lbl.displayFormat = displayFormat
    lbl.showUnits = showUnits
    lbl.alarmSensitiveBorder = alarm_sensitive_border
    lbl.alarmSensitiveContent = alarm_sensitive_content
    lbl.setStyleSheet(TABLE_ALARMS_QSS)
    if precision:
        lbl.precision_changed(precision)
    return lbl



from siriushlacon.epics import create_connected_pv


class ProcessOn:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        self.status1_pv = create_connected_pv(pvname=f"{self.prefix}:ProcessOn:Status1")
        self.status2_pv = create_connected_pv(pvname=f"{self.prefix}:ProcessOn:Status2")
        self.status3_pv = create_connected_pv(pvname=f"{self.prefix}:ProcessOn:Status3")
        self.status4_pv = create_connected_pv(pvname=f"{self.prefix}:ProcessOn:Status4")
        self.status5_pv = create_connected_pv(pvname=f"{self.prefix}:ProcessOn:Status5")

        self.bool = create_connected_pv(pvname=f"{self.prefix}:ProcessOn:Bool")

    def activate_all_status(self):
        self.status1_pv.value = 1
        self.status2_pv.value = 1
        self.status3_pv.value = 1
        self.status4_pv.value = 1
        self.status5_pv.value = 1

    def clear_all_status(self):
        self.status1_pv.value = 0
        self.status2_pv.value = 0
        self.status3_pv.value = 0
        self.status4_pv.value = 0
        self.status5_pv.value = 0

    def toggle(self):
        self.bool.value = 1
        self.bool.value = 0


class ProcessOff:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        self.off_fv_status1_pv = create_connected_pv(
            pvname=f"{prefix}:ProcessOffFV:Status1"
        )
        self.off_fv_status2_pv = create_connected_pv(
            pvname=f"{prefix}:ProcessOffFV:Status2"
        )
        self.off_fv_status3_pv = create_connected_pv(
            pvname=f"{prefix}:ProcessOffFV:Status3"
        )
        self.off_fv_status4_pv = create_connected_pv(
            pvname=f"{prefix}:ProcessOffFV:Status4"
        )
        self.off_fv_status5_pv = create_connected_pv(
            pvname=f"{prefix}:ProcessOffFV:Status5"
        )
        self.off_fv_status6_pv = create_connected_pv(
            pvname=f"{prefix}:ProcessOffFV:Status6"
        )

        self.bool = create_connected_pv(pvname=f"{self.prefix}:ProcessOff:Bool")

    def toggle(self):
        """turn on and off {self.prefix}:Process:Bool PV"""
        self.bool.value = 1
        self.bool.value = 0

    def clear_all_fv_status(self):
        self.off_fv_status1_pv.value = 0
        self.off_fv_status2_pv.value = 0
        self.off_fv_status3_pv.value = 0
        self.off_fv_status4_pv.value = 0
        self.off_fv_status5_pv.value = 0
        self.off_fv_status6_pv.value = 0


class ProcessRecovery:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        # Create process recovery status PVs
        self.status1_pv = create_connected_pv(
            pvname=f"{self.prefix}:ProcessRecovery:Status1"
        )
        self.status2_pv = create_connected_pv(
            pvname=f"{self.prefix}:ProcessRecovery:Status2"
        )
        self.status3_pv = create_connected_pv(
            pvname=f"{self.prefix}:ProcessRecovery:Status3"
        )
        self.status4_pv = create_connected_pv(
            pvname=f"{self.prefix}:ProcessRecovery:Status4"
        )
        self.status5_pv = create_connected_pv(
            pvname=f"{self.prefix}:ProcessRecovery:Status5"
        )

        self.bool = create_connected_pv(pvname=f"{self.prefix}:ProcessRec:Bool")

    def toggle(self):
        self.bool.value = 1
        self.bool.value = 0

    def set_all_clear(self):
        self.status1_pv.value = 0
        self.status2_pv.value = 0
        self.status3_pv.value = 0
        self.status4_pv.value = 0
        self.status5_pv.value = 0


class BBB:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        self.pressure_pv = create_connected_pv(pvname=f"{self.prefix}:BBB:Torr")
        self.pre_vacuum_valve_sw_pv = create_connected_pv(
            pvname=f"{self.prefix}:BBB:Relay1-SW"
        )

        self.gate_valve_sw_pv = create_connected_pv(
            pvname=f"{self.prefix}:BBB:Relay2-SW"
        )
        self.gate_valve_ui_pv = create_connected_pv(
            pvname=f"{self.prefix}:BBB:Relay2-UI"
        )

        self.valve_open_pv = create_connected_pv(pvname=f"{self.prefix}:BBB:ValveOpen")
        self.valve_closed_pv = create_connected_pv(
            pvname=f"{self.prefix}:BBB:ValveClosed"
        )


class Turbovac:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        self.pzd1_sp_tevl_pv = create_connected_pv(
            pvname=f"{self.prefix}:TURBOVAC:PZD1-SP.TEVL"
        )
        self.pzd1_sp_zrvl_pv = create_connected_pv(
            pvname=f"{self.prefix}:TURBOVAC:PZD1-SP.ZRVL"
        )
        self.pzd1_sp_sxvl_pv = create_connected_pv(
            pvname=f"{self.prefix}:TURBOVAC:PZD1-SP.SXVL"
        )

        self.pzd2_rb_pv = create_connected_pv(pvname=f"{self.prefix}:TURBOVAC:PZD2-RB")
        self.pzd2_sp_pv = create_connected_pv(pvname=f"{self.prefix}:TURBOVAC:PZD2-SP")

        self.ak_sp_pv = create_connected_pv(pvname=f"{self.prefix}:TURBOVAC:AK-SP")
        self.ind_sp_pv = create_connected_pv(pvname=f"{self.prefix}:TURBOVAC:IND-SP")
        self.pnu_sp_pv = create_connected_pv(pvname=f"{self.prefix}:TURBOVAC:PNU-SP")
        self.pwe_sp_pv = create_connected_pv(pvname=f"{self.prefix}:TURBOVAC:PWE-SP")

        self.pre_vacuum_valve_sw_pv = create_connected_pv(
            pvname=f"{self.prefix}:BBB:Relay1-SW"
        )

        self.venting_valve_sw_pv = create_connected_pv(
            pvname=f"{self.prefix}:TURBOVAC:VentingValve-SW"
        )
        self.venting_valve_ui_pv_rval = create_connected_pv(
            pvname=f"{self.prefix}:TURBOVAC:VentingValve-UI.RVAL"
        )


class System:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix
        self.off_frequency_pv = create_connected_pv(
            pvname=f"{self.prefix}:SYSTEM:OffFrequency"
        )
        self.off_pressure_base_pv = create_connected_pv(
            pvname=f"{self.prefix}:SYSTEM:OffPressureBase"
        )
        self.off_pressure_exp_pv = create_connected_pv(
            pvname=f"{self.prefix}:SYSTEM:OffPressureExp"
        )

        self.on_pressure_base_pv = create_connected_pv(
            pvname=f"{self.prefix}:SYSTEM:OnPressureBase"
        )
        self.on_pressure_exp_pv = create_connected_pv(
            pvname=f"{self.prefix}:SYSTEM:OnPressureExp"
        )


class ACP:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix

        self.on_off_pv = create_connected_pv(pvname=f"{self.prefix}:ACP:OnOff")
        self.speed_rpm = create_connected_pv(pvname=f"{self.prefix}:ACP:SpeedRPM")



import time

import epics


def create_connected_pv(
    pvname: str,
    callback=None,
    form="time",
    verbose=False,
    auto_monitor=None,
    count=None,
    connection_callback=None,
    connection_timeout=5.0,
    access_callback=None,
    attempt_max: int = 2,
    attempt_delay: float = 0.5,
) -> epics.PV:
    """Create a connected PV or raise an exception"""
    if not pvname:
        raise ValueError("pvname is empty")

    if attempt_delay <= 0:
        attempt_delay = 0.05

    _attempt = 0
    pv = epics.PV(
        pvname=pvname,
        callback=callback,
        form=form,
        verbose=verbose,
        auto_monitor=auto_monitor,
        count=count,
        connection_callback=connection_callback,
        connection_timeout=connection_timeout,
        access_callback=access_callback,
    )

    while _attempt < attempt_max:
        if pv.connect():
            break
        time.sleep(attempt_delay)
        _attempt += 1

    if not pv.connect():
        raise RuntimeError(f"PV={pv} is not connected")
    return pv



from bson.objectid import ObjectId
from qtpy import QtCore


class TableModel(QtCore.QAbstractTableModel):
    data_changed = QtCore.Signal(object, str, ObjectId)

    def __init__(self, data, header):
        super(TableModel, self).__init__()
        self._data = data
        self._header = header

    def setHeader(self, index, new_value):
        self._header[index] = new_value

    def deleteHeader(self, index, new_value):
        self._header.pop(index)
        for data_row in self._data:
            data_row.pop(index)

        self.layoutChanged.emit()

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role in QtCore.Qt.DisplayRole and orientation == QtCore.Qt.Horizontal:
            return self._header[section]

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if role in [QtCore.Qt.DisplayRole, role == QtCore.Qt.EditRole]:
            return self._data[index.row()][index.column()]

    def getRow(self, index):
        return self._data[index.row()]

    def deleteRow(self, index):
        return self._data.pop(index.row())

    def deleteRows(self, indexes):
        indexes = [i.row() for i in indexes]
        self._data = [v for i, v in enumerate(self._data) if i not in indexes]

    def getHeaders(self):
        return self._header

    def setData(self, index, value, role):
        if role == QtCore.Qt.EditRole:
            self._data[index.row()][index.column()] = value
            self.data_changed.emit(
                value, self._header[index.column()], self._data[index.row()][0]
            )
            return True

    def rowCount(self, _):
        return len(self._data)

    def columnCount(self, _):
        return len(self._data[0])

    def flags(self, _):
        return (
            QtCore.Qt.ItemIsSelectable
            | QtCore.Qt.ItemIsEnabled
            | QtCore.Qt.ItemIsEditable
        )



import typing as _typing

from pydm.display import Display as _Display
from qtpy import QtCore as _QtCore
from qtpy.QtWidgets import QDialog as _QDialog
from qtpy.QtWidgets import QGridLayout as _QGridLayout
from qtpy.QtWidgets import QWidget as _QWidget

from siriushlacon import DEFAULT_WINDOW_TITLE as _DEFAULT_WINDOW_TITLE
from siriushlacon.widgets.images import (
    CNPEM_INVISIBLE_LOGO_ICON as _CNPEM_INVISIBLE_LOGO_ICON,
)


class BaseDialog(_QDialog):
    """Base modal dialog"""

    def __init__(
        self,
        parent: _typing.Optional[_QWidget],
        flags: _typing.Union[
            _QtCore.Qt.WindowFlags, _QtCore.Qt.WindowType
        ] = _QtCore.Qt.Dialog,
        window_title: str = _DEFAULT_WINDOW_TITLE,
        macros: _typing.Optional[_typing.Dict[str, str]] = None,
    ) -> None:
        super().__init__(parent=parent, flags=flags)
        self.setModal(True)
        self.setWindowTitle(window_title)
        self.setWindowIcon(_CNPEM_INVISIBLE_LOGO_ICON)
        self.setWindowModality(_QtCore.Qt.WindowModal)
        self.setWindowState(_QtCore.Qt.WindowActive)

        self._display: _typing.Optional[_Display] = None

    def set_display(self, display: _Display):
        if not self.layout() and not self._display:
            self._display = display
            layout = _QGridLayout(self)
            layout.addWidget(self._display, 0, 0)

            self.setLayout(layout)
            self.adjustSize()



import typing

from pydm.widgets import PyDMByteIndicator
from qtpy.QtCore import Qt
from qtpy.QtGui import QColor


def get_byte_indicator(
    parent, content: typing.Optional[str], tooltip: str = "", **kwargs
):
    byte = PyDMByteIndicator(parent, content)
    byte.offColor = QColor(59, 0, 0)
    byte.onColor = QColor(255, 0, 0)
    byte.showLabels = False
    byte.orientation = Qt.Horizontal
    byte.numBits = 12
    return byte



from pydm import Display

from siriushlacon.mks937b.consts import (
    BOOSTER_UI,
    BTS_UI,
    LTB_UI,
    MKS_LAUNCH_UI,
    MKS_OVERVIEW,
    STORAGE_RING_UI,
)
from siriushlacon.widgets.images import CNPEM_PIXMAP, LNLS_PIXMAP


class MksLauncher(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(MksLauncher, self).__init__(parent=parent, args=args, macros=macros)

        self.btnOverview.filenames = [MKS_OVERVIEW]
        self.btnOverview.openInNewWindow = True

        self.btnRing.filenames = [STORAGE_RING_UI]
        self.btnRing.openInNewWindow = True

        self.btnBooster.filenames = [BOOSTER_UI]
        self.btnBooster.openInNewWindow = True

        self.btnLtb.filenames = [LTB_UI]
        self.btnLtb.openInNewWindow = True

        self.btnBts.filenames = [BTS_UI]
        self.btnBts.openInNewWindow = True

        self.label_cnpem.setPixmap(CNPEM_PIXMAP)
        self.label_lnls.setPixmap(LNLS_PIXMAP)

    def ui_filename(self):
        return MKS_LAUNCH_UI

    def ui_filepath(self):
        return MKS_LAUNCH_UI



#!/usr/bin/env python3
import json

import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


DATA_JSON = get_abs_path("data.json")
REGATRON_MAIN = get_abs_path("main.py")
TREE_32 = get_abs_path("tree.py")
COMPLETE_MAIN = get_abs_path("regatron.py")
ALARM_MAIN = get_abs_path("alarm.py")

REGATRON_UI = get_abs_path("ui/main.ui")
TREE_32_UI = get_abs_path("ui/tree32.ui")
COMPLETE_UI = get_abs_path("ui/regatron.ui")
ALARM_UI = get_abs_path("ui/alarm.ui")


def load_data():
    data = None
    with open(DATA_JSON, "r") as f:
        data = json.load(f)
    return data


REGATRON_DEVICES = {}
for d in load_data():
    REGATRON_DEVICES[d["P"]] = {**d}

ERROR_LIST_PDF = get_abs_path("ui/error-list-en.pdf")
CODES = [
    *[str(i) for i in range(10)],
    "A",
    "B",
    "C",
    "D",
    "E",
    "F",
    "G",
    "H",
    "J",
    "K",
    "L",
    "M",
    "N",
    "P",
    "Q",
    "R",
    "S",
    "T",
    "U",
    "V",
    "W",
    "X",
]
READINGS_MAP = {
    # STANDARD_MAP
    0: "Internal",
    1: "Internal (PDSP)",
    2: "Output Current",
    3: "Output Voltage",
    4: "Supply",
    5: "Temperature",
    6: "Communication",
    7: "Internal (Modulator)",
    8: "Internal (AD overrange 1)",
    9: "Internal (AD overrange 2)",
    10: "Internal (AD underrange 1)",
    11: "Internal (AD underrange 2)",
    12: "Login",
    13: "Configuration",
    14: "Configuration 2",
    15: "Miscellaneous",
    # EXTENDED_MAP = {
    16: "IBC System",
    17: "IBC Supply",
    18: "IBC Communication",
    19: "IBC Power",
    20: "IBC Inverter",
    21: "IBC Miscellaneous",
    22: "IBC Inverter 2",
    23: "not used",
    24: "Configuration 4",
    25: "Miscellaneous 2",
    26: "Supply 2",
    27: "Login 2",
    28: "Configuration 3",
    29: "Communication 3",
    30: "Internal 2",
    31: "Communication 2",
}

# READINGS = [
STD_READINGS = [
    "Intrn-Mon",
    "IntrnPDSP-Mon",
    "OutCurrent-Mon",
    "OutVolt-Mon",
    "Supply-Mon",
    "T-Mon",
    "Comm-Mon",
    "IntrnMod-Mon",
    "AD1Ovr-Mon",
    "AD2Ovr-Mon",
    "AD1Undr-Mon",
    "AD2Undr-Mon",
    "Login-Mon",
    "Conf-Mon",
    "Conf2-Mon",
    "Misc-Mon",
]
EXT_READINGS = [
    "IBCSystem-Mon",
    "IBCSuppply-Mon",
    "IBCComm-Mon",
    "IBCPwr-Mon1",
    "IBCInv-Mon",
    "IBCMisc-Mon",
    "IBCInv2-Mon",
    "P-Mon",
    "Conf4-Mon",
    "Misc2-Mon",
    "Supply2-Mon",
    "Login2-Mon",
    "Conf3-Mon",
    "Comm3-Mon",
    "Intrn2-Mon",
    "Comm2-Mon",
]



#!/usr/bin/env python3
import argparse
import os
import subprocess

from siriushlacon.regatron.consts import COMPLETE_MAIN, REGATRON_DEVICES

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"

parser = argparse.ArgumentParser(description="Run SI PS DCLink Interface.")
parser.add_argument("-dev", "--device", type=str, default="")
parser.add_argument("--master", action="store_true")
args = parser.parse_args()

if args.device in REGATRON_DEVICES:
    isMaster = REGATRON_DEVICES[args.device]["master"]
    print("Master", isMaster, type(isMaster))
else:
    isMaster = args.master

subprocess.Popen(
    "pydm --hide-nav-bar -m 'P={}, master={}' {}".format(
        args.device, isMaster, COMPLETE_MAIN
    ),
    shell=True,
)



#!/usr/bin/env python3
import os

from siriushlacon.application import LogLevel, launch_pydm
from siriushlacon.launcher.consts import LAUNCH_WINDOW
from siriushlacon.tools.consts import PYDM_TOOLS_PATH
from siriushlacon.vbc.consts import BBB_IOC_ADDR

if __name__ == "__main__":
    os.environ["EPICS_CA_AUTO_ADDR_LIST"] = "YES"
    os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"
    os.environ["PYDM_TOOLS_PATH"] = PYDM_TOOLS_PATH

    if "EPICS_CA_ADDR_LIST" not in os.environ:
        os.environ["EPICS_CA_ADDR_LIST"] = " ".join(BBB_IOC_ADDR)
    else:
        os.environ["EPICS_CA_ADDR_LIST"] = (
            " ".join(BBB_IOC_ADDR) + os.environ["EPICS_CA_ADDR_LIST"]
        )

    launch_pydm(
        displayfile=LAUNCH_WINDOW,
        hide_nav_bar=True,
        log_level=LogLevel.INFO,
    )



#!/usr/bin/env python
import argparse
import inspect
import json
import os

from siriushlacon.application import LogLevel, launch_pydm
from siriushlacon.launcher.consts import LAUNCH_WINDOW
from siriushlacon.logging import get_logger
from siriushlacon.tools.consts import PYDM_TOOLS_PATH
from siriushlacon.vbc.consts import BBB_IOC_ADDR
from siriushlacon.vbc.consts import MAIN_WINDOW_PY as VBC_MAIN_WINDOW_PY

logger = get_logger("")


def _config_environment():
    os.environ["EPICS_CA_AUTO_ADDR_LIST"] = "YES"
    os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"
    os.environ["PYDM_TOOLS_PATH"] = PYDM_TOOLS_PATH

    if "EPICS_CA_ADDR_LIST" not in os.environ:
        os.environ["EPICS_CA_ADDR_LIST"] = " ".join(BBB_IOC_ADDR)
    else:
        os.environ["EPICS_CA_ADDR_LIST"] = (
            " ".join(BBB_IOC_ADDR) + os.environ["EPICS_CA_ADDR_LIST"]
        )


def launch_generic():
    _config_environment()
    parser = argparse.ArgumentParser("Generic Launcher")
    parser.add_argument(
        "params",
        help=f"json str to be forwarded into `launch_pydm(`{inspect.signature(launch_pydm).__str__()})",
    )
    args = parser.parse_args()
    data = json.loads(args.params)
    logger.info(f"Launch siriushlacon PyDM OPI - {data}")
    launch_pydm(**data)


def launch_main_window():
    _config_environment()
    launch_pydm(
        displayfile=LAUNCH_WINDOW,
        hide_nav_bar=True,
        log_level=LogLevel.INFO,
    )


def launch_vbc():
    parser = argparse.ArgumentParser("VAC Pumps Controller")
    parser.parse_args()
    launch_pydm(
        displayfile=VBC_MAIN_WINDOW_PY, hide_nav_bar=True, log_level=LogLevel.INFO
    )



#!/usr/local/env python3
import os

import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


PYDM_TOOLS_PATH = os.path.dirname(get_abs_path("con_tool.py"))

VIEWER_URL = "https://10.0.38.42/archiver-viewer/index.html"
MGMT_URL = "https://10.0.38.42/mgmt"
BBB_URL = "https://10.0.38.42/bbb-daemon/index.html"



# flake8: noqa
from .check_pressure import check_pressure
from .clear_status import clear_status_off, clear_status_on, clear_status_rec
from .commute_valve import commute_valve, set_venting_valve_state
from .initialization import initialization
from .process_off import process_off
from .process_on import process_on
from .process_recovery import process_recovery



import logging

from pydm import Display
from pydm.widgets.channel import PyDMChannel

from siriushlacon.regatron.consts import READINGS_MAP, TREE_32_UI

logger = logging.getLogger()


def get_report(value, msg):
    erros = []
    for k, v in READINGS_MAP.items():
        if value & (1 << k):
            erros.append(v)
    logger.info("{} {}".format(msg, erros))
    return erros


class AlarmTree(Display):
    def __init__(self, parent=None, macros=None, **kwargs):
        super().__init__(parent=parent, macros=macros, ui_filename=TREE_32_UI)

        self.isSystem = macros.get("D", "Sys") == "Sys"
        self.isWarn = macros.get("T", "Warn") == "Warn"

        # Warning Groups
        self.ch_mod_warn_report = PyDMChannel(
            address="ca://" + macros["P"] + ":ModWarnGroup-Mon",
            value_slot=self.get_mod_warn_report,
        )

        self.ch_sys_warn_report = PyDMChannel(
            address="ca://" + macros["P"] + ":SysWarnGroup-Mon",
            value_slot=self.get_sys_warn_report,
        )

        # Error Groups
        self.ch_mod_error_report = PyDMChannel(
            address="ca://" + macros["P"] + ":ModErrGroup-Mon",
            value_slot=self.get_mod_error_report,
        )

        self.ch_sys_error_report = PyDMChannel(
            address="ca://" + macros["P"] + ":SysErrGroup-Mon",
            value_slot=self.get_sys_error_report,
        )

        if self.isSystem:
            if self.isWarn:
                self.ch_sys_warn_report.connect()
            else:
                self.ch_sys_error_report.connect()
        else:
            if self.isWarn:
                self.ch_mod_warn_report.connect()
            else:
                self.ch_mod_error_report.connect()

    # Warning
    def get_mod_warn_report(self, value):
        self.label.setText("\n".join(get_report(value, "Module")))

    def get_sys_warn_report(self, value):
        self.label.setText("\n".join(get_report(value, "System")))

    # Error
    def get_mod_error_report(self, value):
        self.label.setText("\n".join(get_report(value, "Module")))

    def get_sys_error_report(self, value):
        self.label.setText("\n".join(get_report(value, "System")))



class Severity:
    NO_ALARM = 0
    MINOR_ALARM = 1
    MAJOR_ALARM = 2
    INVALID_ALARM = 3

    @staticmethod
    def nameOf(index):
        return SeverityStrings[index]


class Alarm:
    NO_ALARM = 0
    READ_ALARM = 1
    WRITE_ALARM = 2
    HIHI_ALARM = 3
    HIGH_ALARM = 4
    LOLO_ALARM = 5
    LOW_ALARM = 6
    STATE_ALARM = 7
    COS_ALARM = 8
    COMM_ALARM = 9
    TIMEOUT_ALARM = 10
    HW_LIMIT_ALARM = 11
    CALC_ALARM = 12
    SCAN_ALARM = 13
    LINK_ALARM = 14
    SOFT_ALARM = 15
    BAD_SUB_ALARM = 16
    UDF_ALARM = 17
    DISABLE_ALARM = 18
    SIMM_ALARM = 19
    READ_ACCESS_ALARM = 20
    WRITE_ACCESS_ALARM = 21

    @staticmethod
    def nameOf(index):
        return AlarmStrings[index]


SeverityStrings = ["NO_ALARM", "MINOR", "MAJOR", "INVALID"]

AlarmStrings = [
    "NO_ALARM",
    "READ",
    "WRITE",
    "HIHI",
    "HIGH",
    "LOLO",
    "LOW",
    "STATE",
    "COS",
    "COMM",
    "TIMEOUT",
    "HWLIMIT",
    "CALC",
    "SCAN",
    "LINK",
    "SOFT",
    "BAD_SUB",
    "UDF",
    "DISABLE",
    "SIMM",
    "READ_ACCESS",
    "WRITE_ACCESS",
]



#!/usr/bin/env python3
import os
import subprocess

from siriushlacon.countingpru.consts import GAMMA_COUNTING_MAIN

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"
subprocess.Popen("pydm --hide-nav-bar " + GAMMA_COUNTING_MAIN, shell=True)



#!/bin/bash
import datetime
import os
import platform

import pkg_resources
import pytz


def get_abs_path(relative):
    return pkg_resources.resource_filename(__name__, relative)


SP_TZ = pytz.timezone("America/Sao_Paulo")
TIME_ZERO = datetime.timedelta(0)
ARCHIVER_URL: str = "http://10.0.38.42/retrieval/data/getData.json"

IS_LINUX: bool = os.name == "posix" or platform.system() == "Linux"

# ui
OVERVIEW_UI: str = get_abs_path("ui/overview.ui")

BO, SI, TB, TS = "BO", "SI", "TB", "TS"



import logging
import os
import time

import redis

SERVER_LIST = [
    "10.0.38.59",
    "10.0.38.46",
    "10.0.38.42",
    "10.128.153.81",
    "10.128.153.82",
    "10.128.153.83",
    "10.128.153.84",
    "10.128.153.85",
    "10.128.153.86",
    "10.128.153.87",
    "10.128.153.88",
]
LOG_PATH_SERVER = "bbbread.log"
device = "server"


class Command:
    (
        PING,
        REBOOT,
        EXIT,
        END,
        TYPE,
        APPEND_TYPE,
        REMOVE_TYPE,
        NODE,
        APPEND_NODE,
        REMOVE_NODE,
        SWITCH,
        GET_TYPES,
        GET_UNREG_NODES_SECTOR,
        GET_REG_NODES_SECTOR,
        GET_REG_NODE_BY_IP,
        OK,
        FAILURE,
        SET_IP,
        SET_HOSTNAME,
        SET_NAMESERVERS,
        RESTART_SERVICE,
        STOP_SERVICE,
    ) = range(22)


class RedisServer:
    """Runs on Control System's Server"""

    def __init__(self):
        # Configuring logging
        self.logger = logging.getLogger("bbbreadServer")
        self.logger.setLevel(logging.DEBUG)
        self.logger.debug("Starting up BBBread Server")

        # Probably connecting to a existing server, tries to connect to primary server

        while True:
            for server in SERVER_LIST:
                try:
                    remote_db = redis.StrictRedis(
                        host=server, port=6379, socket_timeout=4
                    )
                    remote_db.ping()
                    self.logger.info("Connected to {} Redis Server".format(server))
                    self.local_db = remote_db
                    return
                except redis.exceptions.ConnectionError:
                    self.logger.warning(
                        "{} Redis server is disconnected".format(server)
                    )
                except redis.exceptions.ResponseError:
                    self.logger.warning(
                        "Could not connect to {}, a response error has ocurred".format(
                            server
                        )
                    )
                    time.sleep(30)
                except Exception as e:
                    self.logger.warning("Could not connect to {}: {}".format(server, e))
                    time.sleep(50)
                continue

            self.logger.info("Server not found. Retrying to connect in 10 seconds...")
            time.sleep(10)

    def get_logs(self, hashname=None):
        if hashname:
            return [
                [key.decode("utf-8"), value.decode("utf-8")]
                for key, value in self.local_db.hgetall(hashname).items()
            ]

        return [name.decode("utf-8") for name in self.local_db.keys("BBB:*:Logs")]

    def list_connected(self, ip="", hostname=""):
        """Returns a list of all BeagleBone Blacks connected to REDIS database
        If IP or hostname is specified lists only the ones with the exact specified parameter"""
        command_instances = []
        log_instances = []
        all_connected = []

        if ip and hostname:
            all_instances = self.local_db.keys("BBB:{}:{}".format(ip, hostname))
        elif ip and not hostname:
            all_instances = self.local_db.keys("BBB:{}:*".format(ip))
            command_instances = self.local_db.keys("BBB:{}:*:Command".format(ip))
            log_instances = self.local_db.keys("BBB:{}:*:Logs".format(ip))
        elif not ip and hostname:
            all_instances = self.local_db.keys("BBB:*:{}".format(hostname))
        else:
            all_instances = self.local_db.keys("BBB:*")
            command_instances = self.local_db.keys("BBB:*:Command")
            log_instances = self.local_db.keys("BBB:*:Logs")

        for node in all_instances:
            if node in command_instances or node in log_instances:
                continue
            all_connected.append(node.decode())
        return all_connected

    def get_node(self, hashname):
        """Returns a BBB info, if an error occurs returns False"""
        try:
            info = self.local_db.hgetall(hashname)
            return info
        except Exception as e:
            self.logger.error("Failed to return nodes info due to error:\n{}".format(e))
            return False

    def send_command(self, ip: str, command, hostname="", override=False):
        """Sends a command to a BeagleBone Black
        Returns False if it fails to send command"""
        try:
            bbb_hashname = self.list_connected(ip, hostname)

            if override and hostname:
                bbb_hashname = ["BBB:{}:{}".format(ip, hostname)]
            if len(bbb_hashname) == 1:
                bbb_state = self.local_db.hget(bbb_hashname[0], "state_string").decode()
                if bbb_state != "Connected":
                    self.logger.error("failed to send command, node is inactive")
                    return False
                bbb_command_listname = "{}:Command".format(bbb_hashname[0])
                check = self.local_db.rpush(bbb_command_listname, command)
                return bool(check)

            if len(bbb_hashname) < 1:
                self.logger.error(
                    "no node found with the specified IP and hostname:" + ip + hostname
                )
            else:
                self.logger.error(
                    "two or more nodes found with the specified ip, please specify a hostname"
                )
            return False
        except Exception as e:
            self.logger.error(
                "A fatal error occurred while sending the command:\n{}".format(e)
            )
            return False

    def bbb_state(self, hashname: str):
        """Verifies if node is active. Ping time inferior to 15 seconds
        Zero if active node, One if disconnected and Two if moved to other hash"""
        now = time.time()

        last_ping = float(self.local_db.hget(hashname, "ping_time").decode())
        time_since_ping = now - last_ping
        node_state = self.local_db.hget(hashname, "state_string").decode()
        last_logs = self.local_db.hvals(hashname + ":Logs")
        if node_state[:3] == "BBB":
            return 2

        if time_since_ping >= 25:
            if node_state != "Disconnected":
                self.local_db.hset(hashname, "state_string", "Disconnected")
                self.log_remote(hashname + ":Logs", "Disconnected", int(now) - 10800)
            return 1

        if last_logs:
            known_status = last_logs[-1].decode()
            if known_status in ["Disconnected", hashname]:
                self.log_remote(hashname + ":Logs", "Reconnected", int(now) - 10800)
        return 0

    def delete_bbb(self, hashname: str):
        """Removes a hash from redis database"""
        self.local_db.delete(hashname)

    def change_hostname(
        self, ip: str, new_hostname: str, current_hostname="", override=False
    ):
        """Changes a BeagleBone Black hostname
        Returns false if an error occurs while sending the command or BBB isn't connected to Redis"""
        command = "{};{}".format(Command.SET_HOSTNAME, new_hostname)
        check = self.send_command(ip, command, current_hostname, override)
        # If command is sent successfully logs hostname change
        if check:
            self.logger.info("{} NEW HOSTNAME - {}".format(ip, new_hostname))
        return check

    def change_nameservers(
        self, ip: str, nameserver_1: str, nameserver_2: str, hostname="", override=False
    ):
        """Changes a BeagleBone Black nameservers
        Returns false if an error occurs while sending the command or BBB isn't connected to Redis"""
        command = "{};{};{}".format(Command.SET_NAMESERVERS, nameserver_1, nameserver_2)
        check = self.send_command(ip, command, hostname, override)
        # If command is sent successfully logs hostname change
        if check:
            self.logger.debug(
                "{} NEW NAMESERVERS - {}  {}".format(ip, nameserver_1, nameserver_2)
            )
        return check

    def change_ip(
        self,
        current_ip: str,
        ip_type: str,
        hostname="",
        new_ip="",
        new_mask="",
        new_gateway="",
        override=False,
    ):
        """Changes a BeagleBone Black IP address (DHCP or manual)
        Returns false if an error occurs while sending the command or BBB isn't connected to Redis"""
        command = "{};{}".format(Command.SET_IP, ip_type)
        if ip_type == "manual":
            # Verifies if new_ip is possible
            check_integrity = new_ip.split(".")
            if len(check_integrity) != 4:
                self.logger.warning("{} NEW IP NOT FORMATTED CORRECTLY")
                return False
            # Verifies if specified IP is available
            ip_available = os.system('ping "-c" 1 -w2 "10.0.6.6" > /dev/null 2>&1')
            if ip_available:
                command += ";{};{};{}".format(new_ip, new_mask, new_gateway)
            else:
                self.logger.warning("{} IP NOT AVAILABLE")
                return False
        check = self.send_command(current_ip, command, hostname, override)
        if check:
            self.logger.info(
                "{} NEW IP - type:{} - new ip: {} - mask: {} - gateway: {}".format(
                    current_ip, ip_type, new_ip, new_mask, new_gateway
                )
            )
        return check

    def reboot_node(self, ip: str, hostname="", override=False):
        """Reboots the specified BeagleBone Black
        Returns false if an error occurs while sending the command or BBB isn't connected to Redis"""
        check = self.send_command(ip, Command.REBOOT, hostname, override)
        if check:
            self.logger.info("{} REBOOT".format(ip))
        return check

    def stop_service(self, ip: str, service: str, hostname="", override=False):
        """Stops the specified service on the given BBB"""
        command = "{};{}".format(Command.STOP_SERVICE, service)
        check = self.send_command(ip, command, hostname, override)
        if check:
            self.logger.info("{} SERVICE STOPPED - {}".format(ip, service))
        return check

    def restart_service(self, ip: str, service: str, hostname="", override=False):
        """Restarts the specified service on the given BBB"""
        command = "{};{}".format(Command.RESTART_SERVICE, service)
        check = self.send_command(ip, command, hostname, override)
        if check:
            self.logger.info("{} SERVICE RESTARTED - {}".format(ip, service))
        return check

    def log_remote(self, bbb, message, date):
        self.local_db.hset(bbb, date, message)



# import multiprocessing
import copy
import enum
import json
import logging
import subprocess
import sys
import typing

import pydm
import pydm.data_plugins
import pydm.utilities
from qtpy.QtWidgets import QMessageBox

from siriushlacon import GENERIC_LAUNCHER_FILE_NAME
from siriushlacon.logging import get_logger

logger = get_logger("")


@enum.unique
class LogLevel(int, enum.Enum):
    DEBUG = logging.DEBUG
    INFO = logging.INFO
    WARNING = logging.WARNING
    ERROR = logging.ERROR
    CRITICAL = logging.CRITICAL


class SiriushlaconApplication(pydm.PyDMApplication):
    def __init__(
        self,
        ui_file=None,
        command_line_args=None,
        display_args=None,
        perfmon=False,
        hide_nav_bar=False,
        hide_menu_bar=False,
        hide_status_bar=False,
        read_only=False,
        macros=None,
        use_main_window=True,
        stylesheet_path=None,
        fullscreen=False,
    ):
        if not command_line_args:
            command_line_args = []
        if not display_args:
            display_args = []
        super().__init__(
            ui_file=ui_file,
            command_line_args=command_line_args,
            display_args=display_args,
            perfmon=perfmon,
            hide_nav_bar=hide_nav_bar,
            hide_menu_bar=hide_menu_bar,
            hide_status_bar=hide_status_bar,
            read_only=read_only,
            macros=macros,
            use_main_window=use_main_window,
            stylesheet_path=stylesheet_path,
            fullscreen=fullscreen,
        )

        self._generic_launcher_file_name = GENERIC_LAUNCHER_FILE_NAME
        self._generic_launcher_file_path = pydm.utilities.which(
            self._generic_launcher_file_name
        )

        if not self._generic_launcher_file_path:
            logger.error(f"'{self._generic_launcher_file_name}' not found by 'which'")

    def new_pydm_process(self, ui_file, macros=None, command_line_args=None):
        kwargs = copy.deepcopy(
            {
                "displayfile": ui_file,
                "macros": macros,
                "hide_nav_bar": self.hide_nav_bar,
                "hide_menu_bar": self.hide_menu_bar,
                "hide_status_bar": self.hide_status_bar,
                "read_only": pydm.data_plugins.is_read_only(),
            }
        )
        kwargs_str = json.dumps(kwargs)
        python_exe = sys.executable

        if not self._generic_launcher_file_path:
            msg = f"Failed to launch pydm process, '{self._generic_launcher_file_name}' not found in path. Using python '{python_exe}'"
            logger.error(msg)
            box = QMessageBox(QMessageBox.Critical, "New PyDM Process Error", msg)
            box.exec()
            return

        logger.info(f"Init New PyDM Processs - {ui_file}")
        logger.info(f"Params: {kwargs_str}")

        subprocess.Popen(
            [self._generic_launcher_file_path, kwargs_str],
            shell=False,
        )


def launch_pydm(
    displayfile: str,
    macros: typing.Union[str, typing.Dict[str, str]] = None,
    hide_nav_bar: bool = True,
    hide_menu_bar: bool = False,
    hide_status_bar: bool = False,
    fullscreen: bool = False,
    read_only: bool = False,
    perfmon: bool = False,
    log_level: LogLevel = LogLevel.INFO,
    *display_args,
    **kwargs,
):
    _display_args = list(display_args)
    if not macros:
        macros = {}

    from pydm.utilities import setup_renderer

    setup_renderer()

    try:
        """
        We must import QtWebEngineWidgets before creating a QApplication
        otherwise we get the following error if someone adds a WebView at Designer:
        ImportError: QtWebEngineWidgets must be imported before a QCoreApplication instance is created
        """
        from qtpy import QtWebEngineWidgets  # noqa: F401
    except ImportError:
        logger.debug("QtWebEngine is not supported.")

    import pydm
    from pydm.utilities.macro import parse_macro_string

    if macros is not None and type(macros) == str:
        macros = parse_macro_string(macros)

    logger.info("siriushlacon PyDM")
    app = SiriushlaconApplication(
        ui_file=displayfile,
        command_line_args=_display_args,
        perfmon=perfmon,
        hide_nav_bar=hide_nav_bar,
        hide_menu_bar=hide_menu_bar,
        hide_status_bar=hide_status_bar,
        fullscreen=fullscreen,
        read_only=read_only,
        macros=macros,
        stylesheet_path=None,
    )

    pydm.utilities.shortcuts.install_connection_inspector(parent=app.main_window)

    # Qt widgets etc.. must be created after the qapplication
    from siriushlacon.widgets.images import CNPEM_INVISIBLE_LOGO_ICON

    app.setWindowIcon(CNPEM_INVISIBLE_LOGO_ICON)
    sys.exit(app.exec_())



#!/usr/bin/env python3
import logging
from typing import List

from conscommon.data import getAgilent
from conscommon.data_model import Device, getBeaglesFromList, getDevicesFromBeagles
from qtpy.QtCore import Qt
from qtpy.QtGui import QColor
from qtpy.QtWidgets import (
    QFrame,
    QGridLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QTableWidget,
    QTableWidgetItem,
)

from siriushlacon.agilent4uhv.tree import DeviceTreeSelection, DeviceTreeView

logger = logging.getLogger()


class DevicesFrame(QFrame):
    def __init__(self, *args, **kwargs):
        super(DevicesFrame, self).__init__(*args, **kwargs)
        self.setFrameStyle(QFrame.Panel | QFrame.Raised)
        self.contentLayout = QGridLayout()

        self.data: List[Device] = []

        # Current Action Status
        self.status = {}

        self.devicePrefixFilterLabel = QLabel(
            "Device filter (Ion Pump not the channel!)"
        )
        self.devicePrefixFilterInp = QLineEdit()
        self.devicePrefixFilterInp.setMaximumWidth(100)

        self.updateDeviceListButton = QPushButton("Apply Filter")
        self.updateDeviceListButton.clicked.connect(self.updateDeviceList)
        self.updateDeviceListButton.setToolTip("Filter the device prefix list.")

        self.contentLayout.addWidget(self.devicePrefixFilterLabel, 0, 0, 1, 2)
        self.contentLayout.addWidget(self.devicePrefixFilterInp, 1, 0, 1, 1)
        self.contentLayout.addWidget(self.updateDeviceListButton, 1, 1, 1, 1)

        self.deviceList = DeviceTreeView()
        self.contentLayout.addWidget(self.deviceList, 2, 0, 2, 2)

        self.deviceStatus = QTableWidget()
        self.deviceStatus.setColumnCount(2)
        # self.deviceStatus.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.deviceStatus.setHorizontalHeaderLabels(["Device", "Status"])
        self.deviceStatusLabel = QLabel("Status")

        self.contentLayout.addWidget(self.deviceStatusLabel, 0, 2, 1, 2)
        self.contentLayout.addWidget(self.deviceStatus, 1, 2, 4, 2)

        self.contentLayout.setRowStretch(2, 2)

        self.setLayout(self.contentLayout)
        self.reloadData()

    def clearStatus(self):
        self.status = {}
        self.deviceStatus.setRowCount(0)

    def updateStatus(self, param):
        self.status[param["device"]] = "{}".format(param["status"])
        idx = 0
        self.deviceStatus.setRowCount(len(self.status))
        for k, v in self.status.items():
            self.deviceStatus.setItem(idx, 0, QTableWidgetItem(k))
            self.deviceStatus.setItem(idx, 1, QTableWidgetItem(v))
            idx += 1

    def getSelectedDevices(self) -> List[DeviceTreeSelection]:
        return self.deviceList.getData()

    def highlightChecked(self, item):
        if item.checkState() == Qt.Checked:
            item.setBackground(QColor("#ffffb2"))
        else:
            item.setBackground(QColor("#ffffff"))

    def updateDeviceList(self):
        devs = []
        _filter = self.devicePrefixFilterInp.text()
        for d in self.data:
            if _filter == "" or _filter in d.prefix:
                devs.append(d)
        self.deviceList.importData(devs)

    def reloadData(self):
        self.data = getDevicesFromBeagles(getBeaglesFromList(getAgilent()))
        self.deviceList.importData(self.data)



#!/usr/bin/env python3
import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


TABLES = ["basic", "advanced", "services", "ps", "logs"]

BEAGLEBONES_MAIN = get_abs_path("main.py")
BEAGLEBONES_MAIN_UI = get_abs_path("ui/main.ui")

INFO_BBB_UI = get_abs_path("ui/infoBBB.ui")
CHANGE_BBB_UI = get_abs_path("ui/configBBB.ui")
LOGS_BBB_UI = get_abs_path("ui/logsBBB.ui")

RED_LED = get_abs_path("ui/Resources/led-red.png")
GREEN_LED = get_abs_path("ui/Resources/led-green.png")



#!/usr/bin/env python3
import logging

from qtpy.QtCore import QThreadPool
from qtpy.QtWidgets import QApplication, QGridLayout, QPushButton, QWidget

from siriushlacon.agilent4uhv.agilent import FIXED as MODE_FIXED
from siriushlacon.agilent4uhv.agilent import STEP as MODE_STEP
from siriushlacon.agilent4uhv.agilent import STEP_TO_FIXED as MODE_STEP_TO_FIXED
from siriushlacon.agilent4uhv.agilent import AgilentAsync, AgilentAsyncRunnable
from siriushlacon.agilent4uhv.devices import DevicesFrame
from siriushlacon.agilent4uhv.parameters import ParametersFrame

logger = logging.getLogger()


class MainWindow(QWidget):
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)

        self.setWindowTitle("VACS - Utility Scripts")

        self.contentLayout = QGridLayout()
        # self.content = QFrame()
        # self.content.setFrameStyle(QFrame.Panel | QFrame.Raised)
        # self.content.setLayout(self.contentLayout)

        # to Step Mode
        self.toStepButton = QPushButton(" to Step ")
        self.toStepButton.setToolTip("Set the voltage behaviour to Step mode.")
        self.toStepButton.clicked.connect(self.toStepAction)
        self.contentLayout.addWidget(self.toStepButton, 0, 0, 1, 1)

        # to Fixed
        self.toFixedButton = QPushButton(" to Fixed ")
        self.toFixedButton.setToolTip(
            "Set the voltage behaviour to Fixed mode and apply a new voltage setpoint."
        )
        self.toFixedButton.clicked.connect(self.toFixedAction)
        self.contentLayout.addWidget(self.toFixedButton, 1, 0, 1, 1)

        # to Step to Fixed
        self.toStepToFixedButton = QPushButton(" to Fixed -> delay -> to Fixed ")
        self.toStepToFixedButton.clicked.connect(self.toStepToFixAction)
        self.toStepToFixedButton.setToolTip(
            "Set the behaviour to Fixed ajusting the channel voltage setpoint, wait n seconds, set the voltage to Fixed applying a new voltage setpoint."
        )
        self.contentLayout.addWidget(self.toStepToFixedButton, 2, 0, 1, 1)

        # Parameters
        self.parameters = ParametersFrame()
        self.parameters.show()
        self.contentLayout.addWidget(self.parameters, 0, 1, 4, 1)

        # Devices
        self.devices = DevicesFrame()
        self.devices.show()
        self.contentLayout.addWidget(self.devices, 5, 0, 1, 2)
        # self.content.show()
        # self.setCentralWidget(self.content)
        self.setLayout(self.contentLayout)

        # Thread !
        self.commandRunning = False

    def debug(self, param):
        self.devices.updateStatus(param)

    def enableComponents(self, enable):
        self.devices.updateDeviceListButton.setEnabled(enable)

        self.toStepButton.setEnabled(enable)
        self.toFixedButton.setEnabled(enable)
        self.toStepToFixedButton.setEnabled(enable)

    def started(self):
        self.commandRunning = True
        self.enableComponents(False)

        self.devices.clearStatus()

    def finished(self):
        self.commandRunning = False
        self.enableComponents(True)

    def toStepAction(self):
        self.doAction(MODE_STEP)

    def toFixedAction(self):
        self.doAction(MODE_FIXED)

    def toStepToFixAction(self):
        self.doAction(MODE_STEP_TO_FIXED)

    def doAction(self, mode):
        if not self.commandRunning:

            agilentAsync = AgilentAsync()
            agilentAsync.timerStatus.connect(self.debug)
            agilentAsync.started.connect(self.started)
            agilentAsync.finished.connect(self.finished)

            runnable = AgilentAsyncRunnable(
                agilentAsync,
                mode=mode,
                voltage=self.parameters.voltage,
                step_to_fixed_delay=self.parameters.delay,
                devices_selection=self.devices.getSelectedDevices(),
            )
            QThreadPool.globalInstance().start(runnable)


def launch():
    app = QApplication([])
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s.%(msecs)03d [%(levelname)s] %(message)s",
        datefmt="%Y-%m-%d,%H:%M:%S",
    )

    window = MainWindow()
    window.show()
    app.exec_()


if __name__ == "__main__":
    launch()



#!/usr/bin/env python3
import argparse
import os
import subprocess

from siriushlacon.beaglebones.consts import BEAGLEBONES_MAIN

parser = argparse.ArgumentParser(description="Beaglebone monitor interface")
parser.add_argument(
    "--redis_host",
    type=str,
    default="",
    help="Redis server IP.",
    required=False,
    dest="redis_host",
)
args = parser.parse_args()

os.environ["PYDM_DEFAULT_PROTOCOL"] = "ca://"
pydm_args = ["--hide-nav-bar", "--hide-menu-bar", "--hide-status-bar"]
subprocess.Popen(
    "pydm {} --hide-nav-bar -m REDIS_HOST={} {}".format(
        " ".join(pydm_args), args.redis_host, BEAGLEBONES_MAIN
    ),
    shell=True,
)



import logging
import multiprocessing
import shlex
import subprocess
import sys
import threading
import typing

from qtpy.QtWidgets import QMainWindow, QWidget

from siriushlacon.utils import close_qt_application

_logger = logging.getLogger(__name__)


class CommandRunner:
    def __init__(
        self,
        command: typing.Callable[[], typing.Any],
        name: typing.Optional[str] = None,
        close_when_finished: bool = False,
        parent_widget: typing.Optional[QWidget] = None,
    ):
        if not command:
            raise RuntimeError("command argument cannot be empty")

        if parent_widget:
            if isinstance(parent_widget, QMainWindow):
                parent_widget = None
                _logger.warning(
                    f"parameter 'parent_widget' {parent_widget} is a {QMainWindow}, arument 'close_when_finished' will terminate the application"
                )
            elif not isinstance(parent_widget, QWidget):
                raise TypeError(
                    f"parameter 'parent_widget' must be of type {QWidget}, received {parent_widget}"
                )
        if not parent_widget and close_when_finished:
            _logger.warning(
                f"{self} does not have a parent widget, qt app will be closed"
            )

        self._parent_widget: typing.Optional[QWidget] = parent_widget
        self._close_when_finished = close_when_finished
        self._command = command
        self._name = name
        self._thread: typing.Optional[
            typing.Union[threading.Thread, multiprocessing.Process]
        ] = None

    def _command_log(self):
        self._command()
        _logger.info(f"Thread finished {self._thread}")
        if self._close_when_finished:
            if self._parent_widget:
                _logger.info(
                    f"qt widget {self._parent_widget} being closed from {self._thread}"
                )
                self._parent_widget.close()
            else:
                _logger.info(f"qt appliacation being closed from {self._thread}")
                close_qt_application()

    def _create_thread(self):
        self._thread = threading.Thread(target=self._command_log, daemon=False)
        if self._name:
            self._thread.name = self._name
        _logger.info(f"Thread '{self._thread}' created")

    def execute_command(
        self, join: bool = False, timeout: typing.Optional[float] = None
    ):
        if self._thread and self._thread.is_alive():
            _logger.error(f"Cannot start thread, '{self._thread}' is alive")
            return

        self._create_thread()
        if not self._thread:
            raise RuntimeError("Failed to create thread")

        self._thread.start()
        _logger.info(f"Thread stated '{self._thread}' join={join}")
        if join:
            self._thread.join(timeout=timeout)


class ShellCommandRunner:
    def __init__(self, command: str):
        self.command = command
        self.process: typing.Optional[subprocess.Popen] = None

    def execute_command(self):
        if not self.command:
            _logger.info("The command is not set, so no command was executed.")
            return

        if self.process is None or self.process.poll() is not None:
            args = shlex.split(self.command, posix="win" not in sys.platform)
            try:
                _logger.info(f"Launching process: {repr(args)}")
                self.process = subprocess.Popen(args)
            except Exception:
                _logger.exception(f"Error in shell command: {self.command}")
        else:
            _logger.error(f"Command '{self.command}' already active.")



#!/usr/bin/env python3

import conscommon.data
import pkg_resources

from siriushlacon.utils import LazyDevices


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


MKS_OVERVIEW = get_abs_path("overview.py")

MKS_MAIN = get_abs_path("main.py")
MKS_MAIN_UI = get_abs_path("ui/table.ui")

MKS_LAUNCH = get_abs_path("launcher.py")
MKS_LAUNCH_UI = get_abs_path("ui/main.ui")

STORAGE_RING = get_abs_path("storage_ring.py")
STORAGE_RING_UI = get_abs_path("ui/storage_ring.ui")

BOOSTER = get_abs_path("booster.py")
BOOSTER_UI = get_abs_path("ui/booster.ui")

BTS = get_abs_path("bts.py")
BTS_UI = get_abs_path("ui/bts.ui")

LTB = get_abs_path("ltb.py")
LTB_UI = get_abs_path("ui/ltb.ui")

NONE_UI = get_abs_path("ui/mks937b/none.ui")

DEVICE_PREVIEW = get_abs_path("device_preview.py")
DEVICE_PREVIEW_UI = get_abs_path("ui/device_preview.ui")

CC_UI = get_abs_path("ui/cc.ui")

PR_UI = get_abs_path("ui/pirani.ui")

PRESSURE = get_abs_path("pressure.py")
PRESSURE_UI = get_abs_path("ui/pressure.ui")

SETTINGS = get_abs_path("settings.py")
SETTINGS_UI = get_abs_path("ui/settings.ui")

INFO_UI = get_abs_path("ui/info.ui")

DEVICE_MENU = get_abs_path("device_menu.py")
DEVICE_MENU_UI = get_abs_path("ui/device_menu.ui")

IOC_MAN_UI = get_abs_path("ui/ioc_man.ui")

lazy_devices = LazyDevices(conscommon.data.getMKS)



#!/usr/bin/env python3
from conscommon.data_model.mks import MKS_SENSOR_COLD_CATHODE
from pydm import Display
from pydm.utilities import IconFont

from siriushlacon.mks937b.consts import DEVICE_MENU, DEVICE_PREVIEW_UI


class DevicePreview(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DevicePreview, self).__init__(parent=parent, args=args, macros=macros)
        self.btn1.filenames = [DEVICE_MENU]
        self.btn1.setIcon(IconFont().icon("edit"))

        if macros:
            if macros.get("A") == MKS_SENSOR_COLD_CATHODE:
                self.PyDMLabel_2.hide()
                self.label_2.hide()

            if macros.get("B") == MKS_SENSOR_COLD_CATHODE:
                self.PyDMLabel_4.hide()
                self.label_4.hide()

            if macros.get("C") == MKS_SENSOR_COLD_CATHODE:
                self.PyDMLabel_6.hide()
                self.label_6.hide()

    def ui_filename(self):
        return DEVICE_PREVIEW_UI

    def ui_filepath(self):
        return DEVICE_PREVIEW_UI



"""Alter REDIS_HOST to your host's IP"""

import subprocess
from datetime import datetime
from time import localtime, sleep, strftime

from pydm import Display
from qtpy import QtCore, QtGui, QtWidgets, uic
from qtpy.QtGui import QBrush
from qtpy.QtWidgets import QInputDialog
from siriuspy.search import PSSearch

from siriushlacon.beaglebones.BBBread import RedisServer
from siriushlacon.beaglebones.consts import (
    BEAGLEBONES_MAIN,
    BEAGLEBONES_MAIN_UI,
    CHANGE_BBB_UI,
    GREEN_LED,
    INFO_BBB_UI,
    LOGS_BBB_UI,
    RED_LED,
    TABLES,
)

# Corporate test server
# REDIS_HOST = '10.0.6.64'
# Sirius server
REDIS_HOST = "10.0.38.59"

room_names = {
    "All": "",
    "Others": "Outros",
    "TL": "LTs",
    "Connectivity": "Conectividade",
    "Power Supplies": "Fontes",
    "RF": "RF",
}
# "LTs", "Conectividade", "Fontes", "RF", "Outros"
for i in range(20):
    room_names["IA-{:02d}".format(i + 1)] = "Sala{:02d}".format(i + 1)

Ui_MainWindow, QtBaseClass = uic.loadUiType(BEAGLEBONES_MAIN_UI)
Ui_MainWindow_config, QtBaseClass_config = uic.loadUiType(CHANGE_BBB_UI)
Ui_MainWindow_info, QtBaseClass_info = uic.loadUiType(INFO_BBB_UI)
Ui_MainWindow_logs, QtBaseClass_logs = uic.loadUiType(LOGS_BBB_UI)


class UpdateNodesThread(QtCore.QThread):
    finished = QtCore.Signal(tuple)

    def __init__(self, server):
        QtCore.QThread.__init__(self)
        self.server = server

    def __del__(self):
        self.wait()

    def run(self):
        nodes = self.server.list_connected()
        nodes_info = {}

        for node in nodes:
            nodes_info[node] = self.server.get_node(node)

        self.finished.emit((nodes, nodes_info))


class UpdateLogsThread(QtCore.QThread):
    finished = QtCore.Signal(list)

    def __init__(self, server, hostname=None):
        QtCore.QThread.__init__(self)
        self.server = server
        self.hostname = hostname

    def __del__(self):
        self.wait()

    def run(self):
        # If no host name is set, all logs must be retrieved
        logs = self.server.get_logs(self.hostname)
        all_logs = logs if self.hostname else []

        # Iterates through BBB logs
        if not self.hostname:
            for name in logs:
                bbb_logs = []
                bbb_logs = self.server.get_logs(name)
                for _log in bbb_logs:
                    _log.extend(name[4 : name.index(":Logs")].split(":", 1))

                all_logs.extend(bbb_logs)

        # Sorts logs by most recent to least recent
        all_logs = sorted(all_logs, key=lambda x: int(x[0]), reverse=True)

        self.finished.emit(all_logs)


class BBBreadMainWindow(Display, QtWidgets.QWidget, Ui_MainWindow):
    """BeagleBone Black Redis Activity Display"""

    def __init__(self, parent=None, macros=None):
        super().__init__(parent=parent, macros=macros, ui_filename=BEAGLEBONES_MAIN_UI)

        # Configures redis Server
        self.server = RedisServer()
        self.sudo = False

        # Table models
        self.logs_model = TableModel([[]], ["Timestamp", "IP", "BBB", "Occurence"])
        self.logsTable.setModel(self.logs_model)

        self.basic_model = TableModel([[]], ["IP", "BBB", "Role", "State"])
        self.basicTable.setModel(self.basic_model)

        self.advanced_model = TableModel([[]], ["IP", "BBB", "Role", "State"])
        self.advancedTable.setModel(self.advanced_model)

        self.services_model = TableModel([[]], ["IP", "BBB", "Role", "State"])
        self.servicesTable.setModel(self.services_model)

        self.ps_model = TableModel([[]], ["IP", "BBB", "Power Supply", "State"])
        self.psTable.setModel(self.ps_model)

        self.servicesTable.horizontalHeader().setResizeMode(
            QtWidgets.QHeaderView.ResizeToContents
        )

        self.basicTable.horizontalHeader().setResizeMode(
            QtWidgets.QHeaderView.ResizeToContents
        )

        self.advancedTable.horizontalHeader().setResizeMode(
            QtWidgets.QHeaderView.ResizeToContents
        )

        # Lists
        self.nodes = []
        self.nodes_info = {}
        self.data = []

        # List Update Timer
        self.autoUpdate_timer = QtCore.QTimer(self)
        self.autoUpdate_timer.timeout.connect(self.update_nodes)
        self.autoUpdate_timer.setSingleShot(False)
        self.autoUpdate_timer.start(1000)

        # Buttons
        self.basicTable.selectionModel().selectionChanged.connect(self.enable_buttons)
        self.advancedTable.selectionModel().selectionChanged.connect(
            self.enable_buttons
        )
        self.servicesTable.selectionModel().selectionChanged.connect(
            self.enable_buttons
        )
        self.logsTable.selectionModel().selectionChanged.connect(self.enable_buttons)
        self.psTable.selectionModel().selectionChanged.connect(self.enable_buttons)
        self.tabWidget.currentChanged.connect(self.enable_buttons)
        self.deleteButton.clicked.connect(self.delete_nodes)
        self.rebootButton.clicked.connect(self.reboot_nodes)
        self.configButton.clicked.connect(self.config_node)
        self.infoButton.clicked.connect(self.show_node_info)
        self.applyserviceButton.clicked.connect(self.service_application)
        self.logsButton.clicked.connect(self.display_logs)
        self.threadCheckBox.stateChanged.connect(self.update_filters)
        self.commandsCheckBox.stateChanged.connect(self.update_filters)

        # Threading
        self.nodes_thread = UpdateNodesThread(self.server)
        self.nodes_thread.finished.connect(self.update_node_list)
        self.logs_thread = UpdateLogsThread(self.server)
        self.logs_thread.finished.connect(self.update_table)

        # Log Filters
        self.toTimeEdit.dateTimeChanged.connect(self.update_filters)
        self.fromTimeEdit.dateTimeChanged.connect(self.update_filters)
        self.filterEdit.textChanged.connect(self.update_filters)

        # Loads loading indicators
        self.loading_icon = QtGui.QPixmap(RED_LED).scaledToHeight(20)
        self.idle_icon = QtGui.QPixmap(GREEN_LED).scaledToHeight(20)

        # State lock
        self.updating = False
        self.fetch_ps = True

    def update_nodes(self):
        """Updates list of BBBs shown"""
        # Stores every BBB information
        self.status_icon.setPixmap(self.loading_icon)
        if not self.nodes_thread.isRunning():
            self.nodes_thread.start()

        # Updates logs
        if not self.logs_thread.isRunning():
            self.logs_thread.start()

    def update_filters(self):
        """Updates tables with filters set by user"""
        if not self.data:
            return

        search = self.filterEdit.text()

        if self.tabWidget.currentIndex() == 4:
            max_date = self.toTimeEdit.dateTime().toPyDateTime().timestamp()
            min_date = self.fromTimeEdit.dateTime().toPyDateTime().timestamp()

            if min_date > max_date:
                self.fromTimeEdit.setDateTime(self.toTimeEdit.dateTime())

            if min_date == max_date:
                self.update_table(self.data)

            length = len(self.data)
            min_index, max_index = length, 0

            # Compares Unix timestamp for logs and filter, stops when a log satisfies the filter
            for index, log in enumerate(self.data):
                if int(log[0]) < min_date:
                    min_index = index
                    break

            for index, log in enumerate(self.data[::-1]):
                if int(log[0]) > max_date:
                    max_index = length - index
                    break

            data = self.data[max_index:min_index]

            if search:
                data = [r for r in data if search in r[3] or search in r[2]]

            self.update_table(data, update=False)
            return True
        elif self.tabWidget.currentIndex() == 3:
            data = {}
            if not self.fetch_ps:
                return False
            for node in self.nodes_info:
                self.nodes_info[node]["ps"] = []
                try:
                    for ps in PSSearch.conv_bbbname_2_psnames(
                        self.nodes_info[node][b"name"].decode().replace("--", ":")
                    ):
                        if search in node or search in ps[0]:
                            self.nodes_info[node]["ps"].append(ps[0])
                            data[node] = self.nodes_info[node]
                except KeyError:
                    continue
                except Exception as e:
                    if "could not" in str(e):
                        self.fetch_ps = False
                        QtWidgets.QMessageBox.question(
                            self,
                            "Error",
                            "Could not connect to control system constants server. Please set the SIRIUS_URL_CONSTS environment variable to a valid server.",
                            QtWidgets.QMessageBox.Ok,
                        )
                    return

            self.update_node_list(data, update=False)
        else:
            data = {}
            for key in self.nodes_info:
                if search in key or search in key.replace("--", ":"):
                    data[key] = self.nodes_info[key]
            self.update_node_list(data, update=False)

    def update_table(self, logs, update=True):
        """Updates content of logs table"""
        if update:
            self.data = logs
            self.update_filters()
            return

        # Formats timestamp in human readable form
        data = [
            [
                datetime.utcfromtimestamp(int(_log[0])).strftime("%d/%m/%Y %H:%M:%S"),
                _log[2],
                _log[3].replace("--", ":"),
                _log[1],
            ]
            for _log in logs
        ]

        # Filters out thread statuses and commands (if boxes aren't checked)
        if self.threadCheckBox.isChecked():
            if not self.commandsCheckBox.isChecked():
                data = [
                    _log
                    for _log in data
                    if "connected" in _log[2].lower()
                    or "hostname" in _log[2].lower()
                    or "thread found an exception" in _log[2].lower()
                    or "thread died" in _log[2].lower()
                ]
        else:
            if not self.commandsCheckBox.isChecked():
                data = [
                    _log
                    for _log in data
                    if "connected" in _log[2].lower() or "hostname" in _log[2].lower()
                ]
            else:
                data = [
                    _log
                    for _log in data
                    if "thread found an exception" not in _log[2].lower()
                    and "thread died" not in _log[2].lower()
                ]

        self.logs_model.set_data(data)

        self.status_icon.setPixmap(self.idle_icon)

    def update_node_list(self, nodes, update=True):  # noqa: C901
        """Gets updated node list and applies it to all lists"""
        if update:
            self.nodes, self.nodes_info = nodes
            self.update_filters()
            return

        names = nodes.keys()
        node_info = nodes
        data = []
        connected_number = 0

        current_tab = self.tabWidget.currentIndex()
        if current_tab == 4:
            return

        if current_tab == 1:
            state_filter = {
                "Connected": self.connectedAdvancedBox.isChecked(),
                "Disconnected": self.disconnectedAdvancedBox.isChecked(),
                "Moved": self.movedAdvancedBox.isChecked(),
            }
        elif current_tab == 0:
            state_filter = {
                "Connected": self.connectedCheckBox.isChecked(),
                "Disconnected": self.disconnectedCheckBox.isChecked(),
                "Moved": self.movedCheckBox.isChecked(),
            }
        elif current_tab == 3:
            state_filter = {"Connected": True, "Disconnected": True, "Moved": True}
        else:
            state_filter = {"Connected": True, "Disconnected": False, "Moved": False}

        list_name = getattr(self, TABLES[current_tab] + "_model")

        # Advanced Tab filters
        ip_filter = {
            "manual": self.staticipAdvancedBox.isChecked(),
            "dhcp": self.dhcpAdvancedBox.isChecked(),
            "Undefined": self.undeterminedAdvancedBox.isChecked(),
            "StaticIP": self.staticipAdvancedBox.isChecked(),
        }
        equipment_filter = {
            "MKS": self.mksAdvancedBox.isChecked(),
            "4UHV": self.uhvAdvancedBox.isChecked(),
            "MBTEMP": self.mbtempAdvancedBox.isChecked(),
            "THERMO": self.thermoAdvancedBox.isChecked(),
            "COUNTING": self.countingpruAdvancedBox.isChecked(),
            "POWER": self.powersupplyAdvancedBox.isChecked(),
            "SPIXCONV": self.spixconvAdvancedBox.isChecked(),
            "SIMAR": self.simarAdvancedBox.isChecked(),
            "Searching": self.nodevAdvancedBox.isChecked(),
            "": self.nodevAdvancedBox.isChecked(),
        }

        self.updating = True
        for node, info in node_info.items():
            if node not in names:
                continue
            try:
                # Organizes node information
                node_ip = info[b"ip_address"].decode()
                node_ip_type = info[b"ip_type"].decode()
                node_name = info[b"name"].decode()
                node_sector = info[b"sector"].decode()
                node_state = info[b"state_string"].decode()
                node_details = info[b"details"].decode()
                node_importance = (
                    info.get(b"matching_bbb").decode().capitalize()
                    if info.get(b"matching_bbb")
                    else "Primary"
                )
            except Exception:
                continue
            # Filters by name and displays node in list
            if room_names[self.roomBox.currentText()] in [node_sector, ""]:
                if node_state[:3] == "BBB":
                    node_state = "Moved"

                if current_tab == 3:
                    if node_state == "Connected":
                        connected_number += 1
                    for ps in info["ps"]:
                        data.append(
                            [node_ip, node_name.replace("--", ":"), ps, node_state]
                        )
                else:
                    for equipment, efilter in equipment_filter.items():
                        # Filters by equipment if advanced tab is selected
                        if (
                            equipment in node_details
                            and efilter
                            and node_ip_type != "0.0.0.0"
                            and (ip_filter[node_ip_type] or ip_filter["Undefined"])
                        ) or current_tab != 1:
                            if state_filter[node_state]:
                                data.append(
                                    [
                                        node_ip,
                                        node_name.replace("--", ":"),
                                        node_importance,
                                        node_state,
                                    ]
                                )
                            break

        list_name.set_data(data)
        self.updating = False
        # Updates the number of connected and listed nodes
        if current_tab == 3:
            self.connectedLabel.setText("Listed PS nodes: {}".format(connected_number))
            self.listedLabel.setText("")
        elif current_tab == 4:
            self.connectedLabel.setText("")
            self.listedLabel.setText("")
        else:
            self.connectedLabel.setText(
                "Connected nodes: {}".format(
                    sum(
                        self.nodes_info[n][b"state_string"].decode() == "Connected"
                        for n in self.nodes_info
                    )
                )
            )
            self.listedLabel.setText("Listed: {}".format(list_name.rowCount()))
        self.status_icon.setPixmap(self.idle_icon)

    @staticmethod
    def remove_faulty(node_string, list_name: QtWidgets.QListWidget, all_elements=True):
        """Removes duplicates and nodes that shouldn't be on the list"""
        qlistitem = list_name.findItems(node_string, QtCore.Qt.MatchExactly)
        if qlistitem:
            if all_elements:
                for node in qlistitem:
                    list_name.takeItem(list_name.row(node))
            elif len(qlistitem) > 1:
                qlistitem.reverse()
                list_name.takeItem(list_name.row(qlistitem[0]))

    def enable_buttons(self):
        """Enables Buttons when one or more boards are selected"""
        current_tab = self.tabWidget.currentIndex()
        selected_items = (
            getattr(self, TABLES[current_tab] + "Table").selectionModel().selectedRows()
        )

        if selected_items:
            self.rebootButton.setEnabled(True)
            self.deleteButton.setEnabled(True)

            is_single = len(selected_items) == 1
            self.configButton.setEnabled(is_single)
            self.infoButton.setEnabled(is_single)
            self.logsButton.setEnabled(is_single)

            self.applyserviceButton.setEnabled(current_tab == 2)
        else:
            self.logsButton.setEnabled(False)
            self.rebootButton.setEnabled(False)
            self.deleteButton.setEnabled(False)
            self.configButton.setEnabled(False)
            self.infoButton.setEnabled(False)
            self.applyserviceButton.setEnabled(False)

    def reboot_nodes(self):
        """Reboots the selected nodes"""
        if not self.sudo:
            text, confirmation = QInputDialog.getText(
                self,
                "Confirmation",
                (
                    "Rebooting could result in downtime, failures or worse, and whoever"
                    "\nexecutes this should be aware of the implications of this action.\n"
                    "\nIf you want to enter sudo mode, type in 'Beaglebone' (case sensitive)"
                ),
            )
            if confirmation and text == "Beaglebone":
                self.sudo = True
            else:
                return

        confirmation = QtWidgets.QMessageBox.question(
            self,
            "Confirmation",
            "Are you sure you want to reboot these nodes?",
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
        )
        if confirmation == QtWidgets.QMessageBox.Yes:
            current_tab = self.tabWidget.currentIndex()
            selected_items = (
                getattr(self, TABLES[current_tab] + "Table")
                .selectionModel()
                .selectedRows()
            )

            for bbb in selected_items:
                bbb_ip = bbb.sibling(bbb.row(), 0 if current_tab != 4 else 1).data()
                bbb_hostname = (
                    bbb.sibling(bbb.row(), 1 if current_tab != 4 else 2)
                    .data()
                    .replace(":", "--")
                )
                self.server.reboot_node(bbb_ip, bbb_hostname)

    def delete_nodes(self):
        """Deletes hashs from Redis Database"""
        confirmation = QtWidgets.QMessageBox.question(
            self,
            "Confirmation",
            "Are you sure about deleting these nodes from Redis Database?",
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
        )
        if confirmation == QtWidgets.QMessageBox.Yes:
            current_tab = self.tabWidget.currentIndex()
            selected_items = (
                getattr(self, TABLES[current_tab] + "Table")
                .selectionModel()
                .selectedRows()
            )

            errors = []
            for bbb in selected_items:
                bbb_ip = bbb.sibling(bbb.row(), 0 if current_tab != 4 else 1).data()
                bbb_hostname = (
                    bbb.sibling(bbb.row(), 1 if current_tab != 4 else 2)
                    .data()
                    .replace(":", "--")
                )

                bbb_hashname = "BBB:{}:{}".format(bbb_ip, bbb_hostname)
                try:
                    self.server.delete_bbb(bbb_hashname)
                    while self.updating:
                        sleep(0.1)
                    self.nodes_info.pop(bbb_hashname)
                    self.update_nodes()

                except KeyError:
                    errors.append(bbb_hashname)
                    continue
            if errors:
                QtWidgets.QMessageBox.warning(
                    self,
                    "Warning",
                    "The following nodes weren't found in the Redis Database:\n{}".format(
                        "\n".join(errors)
                    ),
                    QtWidgets.QMessageBox.Abort,
                )

    def display_logs(self):
        """Shows selected BBB's logs"""
        current_tab = self.tabWidget.currentIndex()

        index = (
            getattr(self, TABLES[current_tab] + "Table")
            .selectionModel()
            .selectedRows()[0]
        )

        bbb_ip = index.sibling(index.row(), 0 if current_tab != 4 else 1).data()
        bbb_hostname = (
            index.sibling(index.row(), 1 if current_tab != 4 else 2)
            .data()
            .replace(":", "--")
        )

        hashname = "BBB:{}:{}:Logs".format(bbb_ip, bbb_hostname)
        try:
            self.window = BBBLogs(self.server, hashname)
            self.window.show()
        except KeyError:
            QtWidgets.QMessageBox.warning(
                self,
                "Warning",
                "The node you are trying to get information isn't connected",
                QtWidgets.QMessageBox.Abort,
            )

    def show_node_info(self):
        """Shows selected BBB's information"""
        current_tab = self.tabWidget.currentIndex()
        index = (
            getattr(self, TABLES[current_tab] + "Table")
            .selectionModel()
            .selectedRows()[0]
        )
        bbb_ip = index.sibling(index.row(), 0 if current_tab != 4 else 1).data()
        bbb_hostname = (
            index.sibling(index.row(), 1 if current_tab != 4 else 2)
            .data()
            .replace(":", "--")
        )

        hashname = "BBB:{}:{}".format(bbb_ip, bbb_hostname)
        try:
            info = self.nodes_info[hashname]
            self.window = BBBInfo(info)
            self.window.show()
        except KeyError:
            QtWidgets.QMessageBox.warning(
                self,
                "Warning",
                "The node you are trying to get information isn't connected",
                QtWidgets.QMessageBox.Abort,
            )

    def config_node(self):
        """Opens configuration the selected BBB's configuration window"""
        current_tab = self.tabWidget.currentIndex()
        index = (
            getattr(self, TABLES[current_tab] + "Table")
            .selectionModel()
            .selectedRows()[0]
        )

        bbb_ip = index.sibling(index.row(), 0 if current_tab != 4 else 1).data()
        bbb_hostname = (
            index.sibling(index.row(), 1 if current_tab != 4 else 2)
            .data()
            .replace(":", "--")
        )

        hashname = "BBB:{}:{}".format(bbb_ip, bbb_hostname)
        info = self.nodes_info[hashname]
        if info[b"state_string"].decode() == "Connected":
            if not self.sudo:
                text, confirmation = QInputDialog.getText(
                    self,
                    "Confirmation",
                    (
                        "This interface is meant for advanced changes that could result in downtime,"
                        "\nfailures or worse.\n"
                        "\nIf you want to enter sudo mode, type in 'Beaglebone' (case sensitive)"
                    ),
                )
                if confirmation and text == "Beaglebone":
                    self.sudo = True

            if self.sudo:
                self.window = BBBConfig(hashname, info, self.server)
                self.window.show()
        else:
            QtWidgets.QMessageBox.warning(
                self,
                "Warning",
                "The node you are trying to configure isn't connected",
                QtWidgets.QMessageBox.Abort,
            )

    def service_application(self):
        """Applies services modification"""
        confirmation = QtWidgets.QMessageBox.question(
            self,
            "Confirmation",
            "Are you sure you want to apply these changes?",
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
        )
        if confirmation == QtWidgets.QMessageBox.Yes:
            index = self.servicesTable.selectionModel().selectedRows()[0]

            selected_operation = self.operationcomboBox.currentText()
            if selected_operation == "Restart":
                operation = self.server.restart_service
            else:
                operation = self.server.stop_service
            selected_bbbs = self.servicesTable.selectionModel().selectedRows()
            for bbb in selected_bbbs:
                bbb_ip = index.sibling(bbb.row(), 0)
                bbb_hostname = index.sibling(bbb.row(), 1)
                if self.bbbreadBox.isChecked():
                    operation(bbb_ip, "bbbread", bbb_hostname)
                if self.bbbfunctionBox.isChecked():
                    operation(bbb_ip, "bbb-function", bbb_hostname)
                if self.ethbridgeBox.isChecked():
                    operation(bbb_ip, "eth-bridge-pru-serial485", bbb_hostname)
                if self.simarBox.isChecked():
                    operation(bbb_ip, "simar_sensors", bbb_hostname)


class BBBInfo(QtWidgets.QWidget, Ui_MainWindow_info):
    """BBB info display"""

    def __init__(self, info):
        QtWidgets.QWidget.__init__(self)
        Ui_MainWindow_info.__init__(self)
        self.setupUi(self)

        if info:
            node_ip = info[b"ip_address"].decode()
            node_ip_type = info[b"ip_type"].decode()
            node_name = info[b"name"].decode()
            node_sector = info[b"sector"].decode()
            ping_time = float(info[b"ping_time"].decode())
            for room, number in room_names.items():
                if number == node_sector:
                    node_sector = room
                    break
            node_state = info[b"state_string"].decode()
            node_details = info[b"details"].decode()
            node_config_time = info[b"config_time"].decode()
            nameservers = info[b"nameservers"].decode()
            self.nameLabel.setText(node_name)
            self.ipLabel.setText(node_ip)
            self.stateLabel.setText(node_state)
            self.iptypeLabel.setText(node_ip_type)
            self.configtimevalueLabel.setText(node_config_time)
            self.equipmentvalueLabel.setText(node_details)
            self.nameserversvalueLabel.setText(nameservers)
            self.sectorvalueLabel.setText(node_sector)
            self.lastseenvalueLabel.setText(
                strftime("%a, %d %b %Y   %H:%M:%S", localtime(ping_time))
            )


class TableModel(QtCore.QAbstractTableModel):
    """Display model for TableView"""

    def __init__(self, data, header):
        super(TableModel, self).__init__()
        self._data = data
        self._header = header

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role == QtCore.Qt.DisplayRole and orientation == QtCore.Qt.Horizontal:
            return self._header[section]

    def data(self, index, role):
        if role in [QtCore.Qt.DisplayRole, QtCore.Qt.EditRole]:
            return self._data[index.row()][index.column()]
        if role == QtCore.Qt.BackgroundRole:
            if self.columnCount() < 3 or self._data[index.row()][2] == "Connected":
                return QBrush(QtCore.Qt.white)
            if self._data[index.row()][3] == "Disconnected":
                return QBrush(QtCore.Qt.red)
            if self._data[index.row()][3] == "Moved":
                return QBrush(QtCore.Qt.yellow)

    def get_data(self):
        return self._data

    def set_data(self, data):
        self._data = data
        self.layoutChanged.emit()

    def rowCount(self, _=None):
        return len(self._data)

    def columnCount(self, _=None):
        if self.rowCount(0) < 1:
            return 0
        return len(self._data[0])


class BBBLogs(QtWidgets.QWidget, Ui_MainWindow_logs):
    # BBB Logs Display
    def __init__(self, server, hashname):
        QtWidgets.QWidget.__init__(self)
        Ui_MainWindow_logs.__init__(self)
        self.setupUi(self)
        self.data = []

        self.logs_thread = UpdateLogsThread(server, hashname)
        self.logs_thread.finished.connect(self.update_table)

        self.model = TableModel([[]], ["Timestamp", "Occurence"])
        self.logsTable.setModel(self.model)

        self.fromTimeEdit.dateTimeChanged.connect(self.update_filters)
        self.toTimeEdit.dateTimeChanged.connect(self.update_filters)

        self.filterEdit.textChanged.connect(self.update_filters)

        self.update_timer = QtCore.QTimer(self)
        self.update_timer.timeout.connect(self.logs_thread.start)
        self.update_timer.setSingleShot(False)
        self.update_timer.start(1000)

    def update_table(self, logs, update=True):
        """Sets table values and converts timestamp, deep copies logs"""
        if update:
            self.data = logs
            self.update_filters()
            return

        data = [
            [
                datetime.utcfromtimestamp(int(_log[0])).strftime("%d/%m/%Y %H:%M:%S"),
                _log[1],
            ]
            for _log in logs
        ]

        self.model.set_data(data)

    def update_filters(self):
        """Updates log table with filters set by user"""
        if not self.data:
            return

        search = self.filterEdit.text()

        max_date = self.toTimeEdit.dateTime().toPyDateTime().timestamp()
        min_date = self.fromTimeEdit.dateTime().toPyDateTime().timestamp()

        if min_date > max_date:
            self.fromTimeEdit.setDateTime(self.toTimeEdit.dateTime())

        if min_date == max_date:
            self.update_table(self.data)

        length = len(self.data)
        min_index, max_index = length, 0

        # Compares Unix timestamp for logs and filter, stops when a log satisfies the filter
        for index, log in enumerate(self.data):
            if int(log[0]) < min_date:
                min_index = index
                break

        for index, log in enumerate(self.data[::-1]):
            if int(log[0]) > max_date:
                max_index = length - index
                break

        data = self.data[max_index:min_index]

        # If the user has set a string filter, all logs without a mention of the filter are removed
        if search:
            data = [r for r in data if search in r[1]]

        self.update_table(data, update=False)


class BBBConfig(QtWidgets.QWidget, Ui_MainWindow_config):
    """BBB configuration display"""

    def __init__(self, hashname, info, server):
        QtWidgets.QWidget.__init__(self)
        Ui_MainWindow_config.__init__(self)
        self.setupUi(self)

        self.server = server

        self.hashname = hashname
        self.hostname = info[b"name"].decode()
        self.ip_address = info[b"ip_address"].decode()
        ip = self.ip_address.split(".")
        self.ip_suffix = ip[-1]
        self.bbb_sector = info[b"sector"].decode()

        self.currenthostnamevalueLabel.setText(self.hostname)
        self.currentipvalueLabel.setText(self.ip_address)

        self.ip_prefix = ".".join(ip[:-1]) + "."

        if ip[1] != "128":
            self.ipComboBox.setEnabled(False)
            self.newipSpinBox.setEnabled(False)
            self.nameserver1Edit.setEnabled(False)
            self.nameserver2Edit.setEnabled(False)
            self.keepipBox.setChecked(True)
            self.keepipBox.setEnabled(False)
            self.keepnameserversBox.setChecked(True)
            self.keepnameserversBox.setEnabled(False)

        self.newipLabel.setText(self.ip_prefix)
        self.ipComboBox.currentIndexChanged.connect(self.disable_spinbox)

        self.applyButton.clicked.connect(self.apply_changes)

    def disable_spinbox(self):
        if self.ipComboBox.currentText() == "MANUAL":
            self.newipSpinBox.setEnabled(True)
        else:
            self.newipSpinBox.setEnabled(False)

    def apply_changes(self):
        # Before asking for confirmation annotates configuration parameters in order to prevent delay bugs
        self.applyButton.setEnabled(False)
        hostname_changed = False
        # DNS
        keep_dns = self.keepnameserversBox.isChecked()
        nameserver_1 = self.nameserver1Edit.text()
        nameserver_2 = self.nameserver2Edit.text()

        # Hostname
        keep_hostname = self.keephostnameBox.isChecked()
        new_hostname = self.hostnameEdit.text()

        # IP
        keep_ip = self.keepipBox.isChecked()
        ip_type = self.ipComboBox.currentText()
        new_ip_suffix = str(self.newipSpinBox.value())

        # Bools to verify if command was sent successfully
        ip_sent = False

        # Confirmation screen
        confirmation = QtWidgets.QMessageBox.question(
            self,
            "Confirmation",
            "Apply changes?",
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
        )
        if confirmation == QtWidgets.QMessageBox.Yes:
            # Nameservers configuration
            if not keep_dns and nameserver_1 and nameserver_2:
                dns_sent = self.server.change_nameservers(
                    self.ip_address, nameserver_1, nameserver_2, self.hostname
                )
            else:
                dns_sent = True
            # Hostname configuration
            if not keep_hostname and new_hostname:
                name_sent = self.server.change_hostname(
                    self.ip_address, new_hostname, self.hostname
                )
                hostname_changed = name_sent
            else:
                name_sent = True
            if not keep_ip:
                if ip_type in ["DHCP", "dhcp"]:
                    if hostname_changed and name_sent:
                        ip_sent = self.server.change_ip(
                            self.ip_address, "dhcp", self.hostname, override=True
                        )
                        self.hostname = new_hostname
                    else:
                        ip_sent = self.server.change_ip(
                            self.ip_address, "dhcp", self.hostname
                        )
                elif new_ip_suffix not in [self.ip_suffix, "0", "1", "2"]:
                    new_ip = self.ip_prefix + new_ip_suffix
                    if hostname_changed and name_sent:
                        ip_sent = self.server.change_ip(
                            self.ip_address,
                            "manual",
                            self.hostname,
                            new_ip,
                            "255.255.255.0",
                            self.ip_prefix + "1",
                            override=True,
                        )
                        self.hostname = new_hostname
                    else:
                        ip_sent = self.server.change_ip(
                            self.ip_address,
                            "manual",
                            self.hostname,
                            new_ip,
                            "255.255.255.0",
                            self.ip_prefix + "1",
                        )
            else:
                ip_sent = True
            if ip_sent and dns_sent and name_sent:
                QtWidgets.QMessageBox.information(
                    self,
                    "Success",
                    "Node configured successfully",
                    QtWidgets.QMessageBox.Close,
                )
            if not name_sent:
                QtWidgets.QMessageBox.warning(
                    self,
                    "Error",
                    "Error in Hostname configuration",
                    QtWidgets.QMessageBox.Abort,
                )
            if not dns_sent:
                QtWidgets.QMessageBox.warning(
                    self,
                    "Error",
                    "Error in Nameservers configuration",
                    QtWidgets.QMessageBox.Abort,
                )
            if not ip_sent:
                QtWidgets.QMessageBox.warning(
                    self,
                    "Error",
                    "Error in IP configuration",
                    QtWidgets.QMessageBox.Abort,
                )
            self.close()
        else:
            self.applyButton.setEnabled(True)


if __name__ == "__main__":
    subprocess.Popen("pydm --hide-nav-bar " + BEAGLEBONES_MAIN, shell=True)



from qtpy.QtGui import QPixmap as _QPixmap

from siriushlacon.vbc.consts import CHECK_IMG as _CHECK_IMG
from siriushlacon.vbc.consts import CNPEM_IMG as _CNPEM_IMG
from siriushlacon.vbc.consts import LNLS_IMG as _LNLS_IMG
from siriushlacon.vbc.consts import PLAY_IMG as _PLAY_IMG
from siriushlacon.vbc.consts import STOP_IMG as _STOP_IMG
from siriushlacon.vbc.consts import WARNING_IMG as _WARNING_IMG

CHECK = _QPixmap(_CHECK_IMG)
CNPEM = _QPixmap(_CNPEM_IMG)
LNLS = _QPixmap(_LNLS_IMG)
PLAY = _QPixmap(_PLAY_IMG)
STOP = _QPixmap(_STOP_IMG)
WARNING = _QPixmap(_WARNING_IMG)



import logging
import time

from siriushlacon.vbc.epics import ACP, BBB, ProcessOff, ProcessOn, System, Turbovac

_logger = logging.getLogger(__name__)


class ProcessOnAction:
    def __init__(self, prefix: str):
        if not prefix:
            raise ValueError(f"parameter prefix cannot be empty {prefix}")
        self.prefix = prefix
        self._tick = 0.5

        self.process_on = ProcessOn(prefix=self.prefix)
        self.process_off = ProcessOff(prefix=self.prefix)
        self.turbovc = Turbovac(prefix=self.prefix)
        self.bbb = BBB(prefix=self.prefix)
        self.acp = ACP(prefix=self.prefix)
        self.system = System(prefix=self.prefix)

    def run(self):
        self._clear_status()
        self._stage_1()
        self._stage_2()
        self._stage_3()
        self._stage_4()
        self._stage_5()

    def _clear_status(self):
        _logger.info("Clear status")
        # clear all status PVs
        self.process_on.clear_all_status()
        self.process_off.clear_all_fv_status()

    def _stage_1(self):
        """wait TURBOVAC pump stops completely"""
        _logger.info("Stage 1")
        self.process_on.status1_pv.value = 1

        while self.turbovc.pzd2_rb_pv.value != 0:
            time.sleep(self._tick)

    def _stage_2(self):
        """Stage 2:"""
        _logger.info("Stage 2")
        # open gate valve (VAT) and the pre-vacuum valve
        self.bbb.gate_valve_sw_pv.value = 1
        self.bbb.pre_vacuum_valve_sw_pv.value = 1

        # update UI checkbox status
        # epics.caput(GATE_VALVE_UI, 1)
        # epics.caput(PRE_VACUUM_VALVE_UI, 1)

        # ---------------------------------------
        # wait gate valve receives command to open
        while self.bbb.pre_vacuum_valve_sw_pv.value == 0:
            time.sleep(self._tick)

        # read gate valve (VAT) status to check if it is really open
        loop = True
        while loop:
            Lo = self.bbb.valve_open_pv.value
            Lg = self.bbb.valve_closed_pv.value

            if Lo & (not (Lg)):
                loop = False
            time.sleep(self._tick)

        self.process_on.status2_pv.value = 1

    def _stage_3(self):
        """Stage 3:"""
        _logger.info("Stage 3")
        # turn ACP15 pump ON
        self.acp.on_off_pv.value = 1
        # wait until pump receives command to turn on
        while self.acp.on_off_pv.value == 0:
            time.sleep(self._tick)

        self.process_on.status3_pv.value = 1

    def check_pressure_limits(self) -> bool:
        """Is pressure under 5*(10^-2) Torr"""
        return self.bbb.pressure_pv.value > (
            self.system.on_pressure_base_pv.value
            * 10 ** self.system.on_pressure_exp_pv.value
        )

    def _stage_4(self):
        """Stage 4: read the pressure and proceed when its value is under 5*(10^-2) Torr"""
        _logger.info("Stage 4")
        while self.check_pressure_limits():
            time.sleep(self._tick)

        self.process_on.status4_pv.value = 1

    def _stage_5(self):
        """Stage 5:"""
        _logger.info("Stage 5")
        # turn TURBOVAC pump ON
        self.turbovc.pzd1_sp_tevl_pv.value = 1
        self.turbovc.pzd1_sp_zrvl_pv.value = 1

        # wait until pump receives command to turn on
        while (self.turbovc.pzd1_sp_tevl_pv.value == 0) & (
            self.turbovc.pzd1_sp_zrvl_pv.value == 0
        ):
            time.sleep(self._tick)

        self.process_on.status5_pv.value = 1

        # complement value of PV to launch "Process Finished" window
        self.process_on.toggle()


def process_on(prefix: str):
    """
    this script do all the procedures to decrease the pressure of the system
    it is divided in 5 stages, described as follow:
        -stage 1: open gate and pre-vacuum valves (and wait gate valve really closes)
        -stage 2: wait until gate valve actually closes
        -stage 3: turn ACP15 on
        -stage 4: wait pressure decrease to less than 0.05 Torr
        -stage 5: turn TURBOVAC on
    """
    action = ProcessOnAction(prefix=prefix)
    action.run()



# ****************Control's Group - MonitoringPRU*******************
#           Author: Robert Willian Polli
#           Last version: 04th April, 2020
# *****************************************************************
import datetime
import hashlib
import sys
import time
import webbrowser
from functools import partial

from epics import PV, caget, caput
from pydm import Display
from qtpy import QtCore, QtWidgets
from qtpy.QtGui import QPixmap

from siriushlacon.countingpru.consts import (
    AFTER_BC_IMAGE,
    BEFORE_BC_IMAGE,
    CNPEM_IMAGE,
    LNLS_IMAGE,
)
from siriushlacon.countingpru.Overview import Overview

qtMainFile = "ui/main.ui"

counters = ["C2", "C3", "M1"]  # Positions of conters on storage ring
Det_Location = ["M2", "C1", "C2", "C3", "C4"]  # Location of detectors

Password = "c67ecf999969e683a24c4fbd9339ca5c"

measurement = ["DayAverage", "WeekAverage", "2WeeksAverage", "MonthAverage"]

link_archiver = "http://10.0.38.42/archiver-viewer/?{}from={}-{:0>2d}-{:0>2d}T{:0>2d}%3A{:0>2d}%3A01.010Z&to={}-{:0>2d}-{:0>2d}T{:0>2d}%3A{:0>2d}%3A01.010Z"


class MonitoringCountingPRU(Display):
    global qtMainFile, counters, Password

    def __init__(self, parent=None, macros=None, args=None):
        super().__init__(
            parent=parent, args=args, macros=macros, ui_filename=qtMainFile
        )

        self.finish_caget = False  # Program's variables
        self.isC2orC4 = False  #

        self.gamma_measurement = {
            "M2": "M1",
            "C1": "M2",
            "C2": "C1",
            "C3": "C2",
            "C4": "C3",
            "M1": "C4",
            "": "",
        }  # {position of sensor:measurement}
        self.Sector_number.valueChanged.connect(
            partial(self.set_box, "counter")
        )  # If value of QComboBox change, call set_box
        self.Archiver_Button.clicked.connect(
            self.openArchiver
        )  # Set function of button
        self.SN.returnPressed.connect(
            self.getPassword
        )  # Called if serial number has been changed
        self.set_box("counter")  # Function to add items to QComboBox

        self.beforeBC_image.setPixmap(QPixmap(BEFORE_BC_IMAGE))
        self.afterBC_image.setPixmap(QPixmap(AFTER_BC_IMAGE))
        self.cnpem_image.setPixmap(QPixmap(CNPEM_IMAGE))
        self.lnls_image.setPixmap(QPixmap(LNLS_IMAGE))

        self.counterBox.currentIndexChanged.connect(
            partial(self.new_counter, True)
        )  # True: chosen on QComboBox, False: Wrote by user
        self.gammaBox.currentIndexChanged.connect(
            partial(self.new_detector, True)
        )  # ""
        self.Counter_name.returnPressed.connect(self.new_counter)  #
        self.Gamma_name.returnPressed.connect(
            self.new_detector
        )  # Called when user search the names

        self.BT_Button.clicked.connect(self.Set_TimeBase)  #
        self.OverviewButton.clicked.connect(self.overview)  # Set functions to bottons
        self.pulsesAverageButton.clicked.connect(self.plot_average)  #
        self.OverviewAverage.clicked.connect(partial(self.plot_average, True))  #

    def plot_average(self, overview=False):
        self.checkBox_list = []
        self.Pulse_channel_Label.setText("Channel = " + self.PV_name_label.text())
        self.LastDayAverage.channel = "ca://{}:DayAverage-Mon".format(
            self.PV_name_label.text()
        )  #
        self.LastWeekAverage.channel = "ca://{}:WeekAverage-Mon".format(
            self.PV_name_label.text()
        )  # Connects PVs of Averages
        self.TwoWeeksAverage.channel = "ca://{}:2WeeksAverage-Mon".format(
            self.PV_name_label.text()
        )  #
        self.LastMonthAverage.channel = "ca://{}:MonthAverage-Mon".format(
            self.PV_name_label.text()
        )  #

        pvs = ""
        for checkBox in range(4):
            if eval("self.checkBox_{}".format(checkBox)).isChecked():
                if overview:
                    Overview(average=measurement[checkBox])  # Opens Average Graphic
                else:
                    pvs += "pv={}%3A{}-Mon&".format(
                        (self.Detector_name).replace(":", "%3A"), measurement[checkBox]
                    )  # Creates the link to archiever
        if pvs != "":
            self.openArchiver(pvs)

    def overview(self):
        overwiew_Window = Overview()
        overwiew_Window.show()

    def openArchiver(self, pv_link=""):
        a = time.gmtime()
        now = datetime.datetime(
            a.tm_year, a.tm_mon, a.tm_mday, a.tm_hour, a.tm_min, a.tm_sec
        )  # Date and time of Greenwich
        new = now - datetime.timedelta(minutes=10)

        if not (pv_link):
            pv = "pv={}%3ACount-Mon&".format((self.Detector_name).replace(":", "%3A"))
        else:
            pv = pv_link

        link = link_archiver.format(
            pv,
            new.year,
            int(new.month),
            int(new.day),
            int(new.hour),
            int(new.minute),
            now.year,
            int(now.month),
            int(now.day),
            int(now.hour),
            int(now.minute),
        )
        webbrowser.open(link)  # Opens the link

    def set_box(
        self, typ
    ):  # Same function to edit QComboBox of Counters and Gamma detectors
        if typ == "counter":
            self.counterBox.clear()
            self.counterBox.addItems([""])
            for i in range(len(counters)):
                if counters[i] != "M1":
                    self.counterBox.addItems(
                        [
                            "SI-{:0>2d}{}:CO-Counter".format(
                                self.Sector_number.value(), counters[i]
                            )
                        ]
                    )
                elif self.Sector_number.value() != 20:
                    self.counterBox.addItems(
                        [
                            "SI-{:0>2d}{}:CO-Counter".format(
                                self.Sector_number.value() + 1, counters[i]
                            )
                        ]
                    )
                else:
                    self.counterBox.addItems(["SI-01{}:CO-Counter".format(counters[i])])

        elif typ == "gamma":
            self.gammaBox.clear()
            if self.Counter_name.text() != "":
                for i in range(1, 7):
                    add = caget(self.Counter_name.text() + ":Ch{}-Cte".format(str(i)))
                    if add != "None":
                        self.gammaBox.addItems([add])
                self.finish_caget = True
                self.new_detector(True)

    def new_counter(self, search=False):  # Show to user counter chosen
        self.finish_caget = False
        if search:
            self.Counter_name.setText(self.counterBox.currentText())
        if self.Counter_name.text() != "":
            self.set_box("gamma")

    def new_detector(self, search=False):  # Show to user gamma sensor chosen
        if search and self.finish_caget and (self.Counter_name.text() != ""):
            if self.Counter_name.text()[5:7] != "M1":  #
                self.Sector_number.setValue(int(self.Counter_name.text()[3:5]))  #
            elif self.Counter_name.text()[3:5] != "01":  # Sets Sector Number Value
                self.Sector_number.setValue(int(self.Counter_name.text()[3:5]) - 1)  #
            else:  #
                self.Sector_number.setValue(int(20))  #
            if self.isC2orC4:
                self.gammaBox.setCurrentIndex(1)
                self.isC2orC4 = False
            self.Gamma_name.setText(self.gammaBox.currentText())
            self.plot_info()

        elif self.finish_caget:  # If user writes the gamma monitor name
            self.Counter_name.clear()
            self.plot_info()  # Finds the name of counter
            if (self.Gamma_name.text()[5:7] == "C2") or (
                self.Gamma_name.text()[5:7] == "C4"
            ):
                self.isC2orC4 = True
            self.set_box("gamma")

    def Set_TimeBase(self):
        if self.checkBox_TB.isChecked():  # Set all time bases
            confirmation = QtWidgets.QMessageBox.question(
                self,
                "Warning",
                "Do you want set all of Time Bases?",
                QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            )
            if confirmation == QtWidgets.QMessageBox.Yes:
                for sector in range(1, 21):
                    for counter in counters:
                        caput(
                            "SI-{:0>2d}{}:CO-Counter:TimeBase-SP".format(
                                sector, counter
                            ),
                            self.BT_SpinBox.value(),
                        )

        elif self.Counter_name.text() != "":
            confirmation = QtWidgets.QMessageBox.question(
                self,
                "Warning",
                "Are you sure?",
                QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            )
            if confirmation == QtWidgets.QMessageBox.Yes:
                caput(
                    self.Counter_name.text() + ":TimeBase-SP", self.BT_SpinBox.value()
                )
                self.plot_info()
        else:
            QtWidgets.QMessageBox.warning(self, "Attention", "Please, select a counter")

    def getPassword(self):
        text, ok = QtWidgets.QInputDialog.getText(
            self, "Attention", "Password?", QtWidgets.QLineEdit.Password
        )
        if ok:
            if hashlib.md5(text.encode("utf-8")).hexdigest() == Password:
                caput(self.gamaselected + ":SerialNumber-Cte", int(self.SN.text()))
                QtWidgets.QMessageBox.information(
                    self, "Changed", "Gamma sensor Serial Number changed"
                )
            else:
                QtWidgets.QMessageBox.warning(self, "Attention", "Password Incorrect")
        self.SN.setText(str(caget(self.gamaselected + ":SerialNumber-Cte")))

    def plot_info(self, value=""):
        if self.Counter_name.text() == "" and self.Gamma_name.text() == "":
            QtWidgets.QMessageBox.warning(
                self, "Attention", "Please, select a counter or detector"
            )
            return ()

        if self.Counter_name.text() == "":
            try:
                if (
                    self.Gamma_name.text()[-8:] == "CO-Gamma"
                ):  # Search by measurement, 'XXX:CO-Gamma'
                    self.Detector_name = self.Gamma_name.text()
                    Counter = caget(self.Gamma_name.text() + ":Counter-Cte")
                    self.Sector_number.setValue(int(self.Gamma_name.text()[3:5]))

                    self.Counter_name.setText(Counter[:-4])
                    self.Gamma_name.setText(
                        caget(self.Gamma_name.text() + ":Sensor-Cte")
                    )

                elif (
                    self.Gamma_name.text()[-13:] == "GammaDetector"
                ):  # Search by detector name 'XXX:GammaDetector'
                    if self.Gamma_name.text()[5:7] == "M1":
                        if self.Gamma_name.text()[3:5] != "01":
                            self.Detector_name = "SI-{:0>2d}{}{}".format(
                                int(self.Gamma_name.text()[3:5]) - 1,
                                self.gamma_measurement[self.Gamma_name.text()[5:7]],
                                self.Gamma_name.text()[7:16],
                            )
                        else:
                            self.Detector_name = "SI-20{}{}".format(
                                self.gamma_measurement[self.Gamma_name.text()[5:7]],
                                self.Gamma_name.text()[7:16],
                            )
                    else:
                        self.Detector_name = (
                            self.Gamma_name.text()[:5]
                            + self.gamma_measurement[self.Gamma_name.text()[5:7]]
                            + self.Gamma_name.text()[7:16]
                        )
                    self.Sector_number.setValue(int(self.Detector_name[3:5]))
                    self.Counter_name.setText(
                        caget(self.Detector_name + ":Counter-Cte")[:-4]
                    )
                else:
                    return ()
            except Exception:
                QtWidgets.QMessageBox.warning(
                    self, "Attention", "Please, choose a valid gamma detector"
                )
                return ()
        else:
            self.Detector_name = "SI-{:0>2d}{}:CO-Gamma".format(
                self.Sector_number.value(),
                self.gamma_measurement[self.Gamma_name.text()[5:7]],
            )

        self.SN.setEnabled(True)
        self.Archiver_Button.setEnabled(True)
        self.pulsesAverageButton.setEnabled(True)

        self.list_Channels.clear()
        self.PV_name_label.setText(self.Detector_name)
        self.Label_Counter.setText(self.Counter_name.text())

        self.LC.channel = "ca://{}:Count-Mon".format(self.Detector_name)  #
        self.TB.channel = "ca://{}:TimeBase-SP".format(
            self.Counter_name.text()
        )  # Connect to PVs of Counting and Time Base

        for sensors in range(1, 7):  # Finds which channels are being used
            ins = QtWidgets.QListWidgetItem()
            pv = PV(self.Counter_name.text() + ":Ch{}-Cte".format(sensors))
            time.sleep(0.1)
            if not (pv.connected):
                QtWidgets.QMessageBox.warning(
                    self,
                    "Warning",
                    "PV {} is not connected".format(
                        self.Counter_name.text() + ":Ch{}-Cte".format(sensors)
                    ),
                )
            else:
                ins.setText(
                    caget(self.Counter_name.text() + ":Ch{}-Cte".format(sensors))
                )
                if ins.text() == self.Gamma_name.text():
                    self.gamaselected = ins.text()
                    ins.setBackground(QtCore.Qt.cyan)

                elif self.Gamma_name.text()[-13:] != "GammaDetector":
                    if self.Gamma_name.text() == "":
                        return ()
                    if ins.text() == caget(self.Gamma_name.text() + ":Sensor-Cte"):
                        self.gamaselected = ins.text()
                        ins.setBackground(QtCore.Qt.cyan)
                self.list_Channels.addItem(ins)
                self.image_setting()

        self.SN.setText(
            str(caget(self.gamaselected + ":SerialNumber-Cte"))
        )  # Sets value of Serial Number

    def image_setting(
        self,
    ):  # Changes the images to show which counter and detector are being used
        for gammaclear in ("C1", "C2", "C3", "C4", "M1"):
            eval("self.bit_Gamma{}".format(gammaclear)).brush = QtCore.Qt.lightGray
        for counterclear in counters:
            eval("self.bit_Counter{}".format(counterclear)).brush = QtCore.Qt.lightGray
        eval("self.bit_Gamma{}".format(self.gamaselected[5:7])).brush = QtCore.Qt.green
        eval(
            "self.bit_Counter{}".format(self.Counter_name.text()[5:7])
        ).brush = QtCore.Qt.red


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    window = MonitoringCountingPRU()
    window.show()
    sys.exit(app.exec_())



import logging as _logging
import typing as _typing

_formatter = _logging.Formatter(
    "[%(asctime)s %(levelname)s %(filename)s:%(lineno)s - %(funcName)s] %(message)s"
)
_console_handler = _logging.StreamHandler()
_console_handler.setFormatter(_formatter)


def get_logger(
    name=__file__,
    level: int = _logging.INFO,
    handlers: _typing.Optional[_typing.List[_logging.Handler]] = None,
) -> _logging.Logger:
    """Returns a logger object"""

    logger = _logging.getLogger(name)

    if not handlers:
        logger.setLevel(level)
        logger.addHandler(_console_handler)
    return logger



__version__ = "2.4.2"
__author__ = "Claudio F. Carneiro"
__date__ = "Fri, 27 Aug 2021 16:36:41 +0000"

DEFAULT_WINDOW_TITLE: str = "Sirius Interfaces"
GENERIC_LAUNCHER_FILE_NAME: str = "sirius-hla-as-ap-generic-launcher"
VBC_LAUNCHER_FILE_NAME: str = "sirius-hla-as-va-vbc"
MAIN_LAUNCHER_FILE_NAME: str = "sirius-hla-as-ap-conlauncher"



import pkg_resources


def get_abs_path(filename):
    return pkg_resources.resource_filename(__name__, filename)


LAUNCH_WINDOW = get_abs_path("launcher.py")
LAUNCH_WINDOW_UI = get_abs_path("ui/launcher.ui")

PCTRL_MAIN = get_abs_path("../pctrl/main.py")



import datetime
import json
import logging

import epics
from pydm import Display
from pydm.utilities import IconFont
from pydm.widgets.channel import PyDMChannel
from qtpy.QtCore import QUrl
from qtpy.QtGui import QDesktopServices
from qtpy.QtWidgets import QTableWidgetItem

from siriushlacon.regatron.consts import (
    ALARM_MAIN,
    CODES,
    COMPLETE_UI,
    ERROR_LIST_PDF,
    READINGS_MAP,
    TREE_32,
)

logger = logging.getLogger()


class ProtectionLevel(object):
    OPERATION = 0
    ADVANCED = 1


class TabId(object):
    Overview = 0
    System = 1
    Module = 2
    ErrWarn = 3
    Advanced = 4


class Regatron(Display):
    def __init__(self, parent=None, macros=None, **kwargs):
        super().__init__(parent=parent, macros=macros, ui_filename=COMPLETE_UI)
        self.macros_dict = macros
        self.isMaster = macros.get("master", "1") == "1"

        self.protectionLevel: ProtectionLevel = ProtectionLevel.OPERATION
        self.setup_icons()

        self.btnModHistory.filenames = [ALARM_MAIN]
        self.btnModHistory.macros = [json.dumps({"T": "Mod", "P": macros["P"]})]
        self.btnModHistory.openInNewWindow = True

        self.btnSysHistory.filenames = [ALARM_MAIN]
        self.btnSysHistory.macros = [json.dumps({"T": "Sys", "P": macros["P"]})]
        self.btnSysHistory.openInNewWindow = True

        self.btnModWarn.filenames = [TREE_32]
        self.btnSysWarn.filenames = [TREE_32]
        self.btnModErr.filenames = [TREE_32]
        self.btnSysErr.filenames = [TREE_32]

        self.btnModWarn.macros = json.dumps(
            {"Title": "Module Warnings", "P": macros["P"], "D": "Mod", "T": "Warn"}
        )
        self.btnSysWarn.macros = json.dumps(
            {"Title": "System Warnings", "P": macros["P"], "D": "Sys", "T": "Warn"}
        )
        self.btnModErr.macros = json.dumps(
            {"Title": "Module Error", "P": macros["P"], "D": "Mod", "T": "Err"}
        )
        self.btnSysErr.macros = json.dumps(
            {"Title": "System Error", "P": macros["P"], "D": "Sys", "T": "Err"}
        )

        # ----------------------------------------------

        self.btnProtectionLevel.passwordProtected = True
        self.btnProtectionLevel.password = "ELPS"
        self.btnProtectionLevel.clicked.connect(self.changeProtectionLevel)
        self.btnProtectionLevel.setText(
            "OPERATION"
            if self.protectionLevel == ProtectionLevel.OPERATION
            else "ADVANCED"
        )
        self.changeItemProtection(
            unlock=True if self.protectionLevel == ProtectionLevel.ADVANCED else False
        )

        self.lblTitle.setText(
            "{} - {}".format(
                macros["P"], "Master" if macros.get("master", "1") == "1" else "slave"
            )
        )
        self.configWidget()

        self.slopeVoltageMax = PyDMChannel(
            address="ca://" + macros["P"] + ":SysSlopeVoltMax-Mon",
            value_slot=self.voltageSlopeMaxChanged,
        )
        self.slopeVoltageMin = PyDMChannel(
            address="ca://" + macros["P"] + ":SysSlopeVoltMin-Mon",
            value_slot=self.voltageSlopeMinChanged,
        )
        self.slopeVoltageMax.connect()
        self.slopeVoltageMin.connect()

        self.slopeCurrentMax = PyDMChannel(
            address="ca://" + macros["P"] + ":SysSlopeCurrMax-Mon",
            value_slot=self.currentSlopeMaxChanged,
        )
        self.slopeCurrentMin = PyDMChannel(
            address="ca://" + macros["P"] + ":SysSlopeCurrMin-Mon",
            value_slot=self.currentSlopeMinChanged,
        )
        self.slopeCurrentMax.connect()
        self.slopeCurrentMin.connect()

        self.modStatusPV = PyDMChannel(
            address="ca://{}{}".format(macros["P"], ":ModState-Mon"),
            value_slot=self.update_mod_state_label,
        )
        self.sysStatusPV = PyDMChannel(
            address="ca://{}{}".format(macros["P"], ":OpMode-Sts"),
            value_slot=self.update_sys_state_label,
        )
        self.modStatusPV.connect()
        self.sysStatusPV.connect()

        # Initial reading from epics.PV
        try:
            self.flash_error_history(self.errorHistoryPV.value)
            # self.update_sys_state_labels(self.sysStatusPV.char_value)
            # self.update_mod_state_labels(self.modStatusPV.value)
        except Exception:
            pass

        # --------- Power Up and Operating time ------------
        self.oprTimeChannel = PyDMChannel(
            address="ca://{}{}".format(macros["P"], ":OperatingTime-Mon"),
            value_slot=self.opr_time_update,
        )
        self.pwrupTimeChannel = PyDMChannel(
            address="ca://{}{}".format(macros["P"], ":PowerUpTime-Mon"),
            value_slot=self.prwup_time_update,
        )
        self.operatingTimeDatetime = None
        self.pwrupTimeDatetime = None

        self.oprTimeChannel.connect()
        self.pwrupTimeChannel.connect()

        # ------------ Flash Error History
        # @todo: do this better !
        self.flashHistoryTable.verticalHeader().setVisible(False)

        self.errorHistoryPV = epics.PV(
            pvname="{}{}".format(macros["P"], ":ErrorHistory-Mon"),
            callback=self.flash_error_history,
        )
        # ------------- DOCS ------------
        self.btnErrorsDocs.clicked.connect(
            lambda self: QDesktopServices.openUrl(QUrl.fromLocalFile(ERROR_LIST_PDF))
        )

    def opr_time_update(self, value):
        self.operatingTimeDatetime = datetime.timedelta(seconds=value)
        self.lblOperatingTime.setText(str(self.operatingTimeDatetime))
        self.flash_error_history(value=self.errorHistoryPV.value)

    def prwup_time_update(self, value):
        self.pwrupTimeDatetime = datetime.timedelta(seconds=value)
        self.lblPowerupTime.setText(str(self.operatingTimeDatetime))
        self.flash_error_history(value=self.errorHistoryPV.value)

    ###     self.oprTimeChannel = epics.PV(
    ###         pvname="{}{}".format(macros["P"], ":OperatingTime-Mon"),
    ###         callback=self.opr_time_update,
    ###     )
    ###     self.pwrupTimeChannel = epics.PV(
    ###         pvname="{}{}".format(macros["P"], ":PowerUpTime-Mon"),
    ###         callback=self.prwup_time_update,
    ###     )

    ### def opr_time_update(self, *args, **kwargs):
    ###     value = kwargs["value"]
    ###     self.lblOperatingTime.setText(str(datetime.timedelta(seconds=value)))

    ### def prwup_time_update(self, *args, **kwargs):
    ###     value = kwargs["value"]
    ###     self.lblPowerupTime.setText(str(datetime.timedelta(seconds=value)))
    def get_char_val(self, val):
        char_value = ""
        if val == 2:
            char_value = "POWERUP"
        elif val == 4:
            char_value = "READY"
        elif val == 8:
            char_value = "RUN"
        elif val == 10:
            char_value = "WARN"
        elif val == 12:
            char_value = "ERROR"
        elif val == 15:
            char_value = "STOP"
        return char_value

    def get_style_from_state(self, state):

        color = "white"
        bgColor = "gray"

        if state in ["POWERUP", "READY"]:
            bgColor = "darkGreen"
        elif state == "RUN":
            bgColor = "green"
        elif state == "WARN":
            bgColor = "yellow"
        elif bgColor == "ERROR":
            bgColor = "RED"

        return """color: {}; background-color: {};""".format(color, bgColor)

    def update_mod_state_label(self, val):
        char_value = self.get_char_val(val)
        self.modStatus.setText(char_value)
        self.modStatus.setStyleSheet(self.get_style_from_state(char_value))

    def update_sys_state_label(self, val):
        char_value = self.get_char_val(val)
        self.sysStatus.setText(char_value)
        self.sysStatus.setStyleSheet(self.get_style_from_state(char_value))

    def get_group_string(self, group_):
        group = int(group_)

        for k, v in READINGS_MAP.items():
            if (group >> k) == 1:
                return "{}) {}".format(CODES[k], v)
        return "{:X}) ?".format(group)

    def get_code_string(self, code_):
        code = int(code_)

        for k in range(32):
            if (code >> k) == 1:
                return "{}".format(CODES[k])
        return "{}?".format(code)

    def flash_error_history(self, *args, **kwargs):
        if not kwargs["value"]:
            return

        self.flashHistoryTable.clearContents()
        values = kwargs["value"][:-1]

        self.flashHistoryTable.setRowCount(len(values))
        i = 0
        for value in values:
            n, day, hour, minute, second, milli, group, code = value.split(",")
            eventTimedelta = datetime.timedelta(
                days=float(day),
                hours=float(hour),
                seconds=float(second),
                milliseconds=float(milli),
            )
            self.flashHistoryTable.setItem(i, 0, QTableWidgetItem(n))
            self.flashHistoryTable.setItem(
                i,
                1,
                QTableWidgetItem(str(eventTimedelta)),
            )

            if self.operatingTimeDatetime and self.pwrupTimeDatetime:
                self.flashHistoryTable.setItem(
                    i,
                    2,
                    QTableWidgetItem(
                        str(
                            datetime.datetime.now()
                            - (self.operatingTimeDatetime - eventTimedelta)
                        )
                        + " + timeoffset"
                    ),
                )

            self.flashHistoryTable.setItem(
                i, 3, QTableWidgetItem(self.get_group_string(group))
            )
            self.flashHistoryTable.setItem(
                i, 4, QTableWidgetItem(self.get_code_string(code))
            )
            i += 1

    def voltageSlopeMaxChanged(self, value):
        self.spbxSlopeVoltSp.setMaximum(value)
        self.spbxSlopeStartupVoltSp.setMaximum(value)

    def voltageSlopeMinChanged(self, value):
        self.spbxSlopeVoltSp.setMinimum(value)
        self.spbxSlopeVoltSp.setSingleStep(value)

        self.spbxSlopeStartupVoltSp.setMinimum(value)
        self.spbxSlopeStartupVoltSp.setSingleStep(value)

    def currentSlopeMaxChanged(self, value):
        self.spbxSlopeCurrSp.setMaximum(value)
        self.spbxSlopeStartupCurrSp.setMaximum(value)

    def currentSlopeMinChanged(self, value):
        self.spbxSlopeCurrSp.setMinimum(value)
        self.spbxSlopeCurrSp.setSingleStep(value)

        self.spbxSlopeStartupCurrSp.setMinimum(value)
        self.spbxSlopeStartupCurrSp.setSingleStep(value)

    def configWidget(self):
        self.tabWidget.setTabEnabled(TabId.System, False)
        self.sysFrame.setVisible(self.isMaster)
        self.btnSysErr.setVisible(self.isMaster)
        self.btnSysWarn.setVisible(self.isMaster)
        self.btnClear.setVisible(self.isMaster)
        self.btnSave.setVisible(self.isMaster)

        try:
            # This function was introduced in Qt 5.15.
            self.tabWidget.setTabVisible(TabId.System, self.isMaster)
        except Exception:
            pass

        self.sysOutVolt.setVisible(self.isMaster)
        self.sysOutCurr.setVisible(self.isMaster)
        self.sysOutPwr.setVisible(self.isMaster)
        self.sysStatus.setVisible(self.isMaster)

        self.warnByte.channel = "ca://{}{}".format(
            self.macros_dict["P"],
            ":GenWarn-Mon" if self.isMaster else ":ModWarnGroup-Mon",
        )
        self.errorByte.channel = "ca://{}{}".format(
            self.macros_dict["P"],
            ":GenIntlk-Mon" if self.isMaster else ":ModErrGroup-Mon",
        )

    def changeProtectionLevel(self):

        if (
            self.protectionLevel == ProtectionLevel.OPERATION
            and self.btnProtectionLevel.validate_password()
        ):
            self.protectionLevel = ProtectionLevel.ADVANCED

        elif self.protectionLevel == ProtectionLevel.ADVANCED:
            # Lock things up ! No need for password prompt
            self.protectionLevel = ProtectionLevel.OPERATION

        self.btnProtectionLevel.setText(
            "OPERATION"
            if self.protectionLevel == ProtectionLevel.OPERATION
            else "ADVANCED"
        )
        self.changeItemProtection(
            unlock=True if self.protectionLevel == ProtectionLevel.ADVANCED else False
        )

    def changeItemProtection(self, unlock):
        if self.isMaster:
            self.btnSave.setEnabled(unlock)

            self.leSysVoltRefSp.setEnabled(unlock)
            self.leSysCurrRefSp.setEnabled(unlock)
            self.leSysPwrRefSp.setEnabled(unlock)
            self.leSysResRefSp.setEnabled(unlock)

            self.btnSave.setVisible(unlock)
            self.leSysVoltRefSp.setVisible(unlock)
            self.leSysCurrRefSp.setVisible(unlock)
            self.leSysPwrRefSp.setVisible(unlock)
            self.leSysResRefSp.setVisible(unlock)
            self.tabWidget.setTabEnabled(TabId.System, unlock)

        self.tabWidget.setTabEnabled(TabId.Module, unlock)
        self.tabWidget.setTabEnabled(TabId.ErrWarn, unlock)
        self.tabWidget.setTabEnabled(TabId.Advanced, unlock)

    def setup_icons(self):
        REFRESH_ICON = IconFont().icon("refresh")
        # Overview
        self.btnSstate.setIcon(REFRESH_ICON)
        self.btnSCtrlMode.setIcon(REFRESH_ICON)
        self.btnMState.setIcon(REFRESH_ICON)
        self.btnMCtrlMode.setIcon(REFRESH_ICON)
        self.btnActIFace.setIcon(REFRESH_ICON)

        self.btnSave.setIcon(IconFont().icon("download"))
        self.btnClear.setIcon(IconFont().icon("eraser"))

        # Module
        # System
        self.btnSysVoltRef.setIcon(REFRESH_ICON)
        self.btnSysCurrRef.setIcon(REFRESH_ICON)
        self.btnSysResRef.setIcon(REFRESH_ICON)
        self.btnSysPwrRef.setIcon(REFRESH_ICON)
        self.btnSysVoltRef_2.setIcon(REFRESH_ICON)
        self.btnSysCurrRef_2.setIcon(REFRESH_ICON)
        self.btnSysResRef_2.setIcon(REFRESH_ICON)
        self.btnSysPwrRef_2.setIcon(REFRESH_ICON)

        self.btnVoltSlope.setIcon(REFRESH_ICON)
        self.btnCurrSlope.setIcon(REFRESH_ICON)

        # Advanced



#!/usr/bin/env python3
import json

from conscommon.data_model.mks import MKS_SENSOR_COLD_CATHODE, MKS_SENSOR_PIRANI
from pydm import Display

from siriushlacon.mks937b.consts import CC_UI, PR_UI, SETTINGS_UI
from siriushlacon.mks937b.macros import get_macro


class Settings(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(Settings, self).__init__(parent=parent, args=args, macros=macros)

        self.pdispA.macros = json.dumps(
            get_macro(macros.get("DEVICE"), "A", macros.get("G1"), macros.get("G2"))
        )
        self.pdispB.macros = json.dumps(
            get_macro(macros.get("DEVICE"), "B", macros.get("G3"), macros.get("G4"))
        )
        self.pdispC.macros = json.dumps(
            get_macro(macros.get("DEVICE"), "C", macros.get("G5"), macros.get("G6"))
        )

        if macros:
            if macros.get("A") == MKS_SENSOR_COLD_CATHODE:
                self.pdispA.filename = CC_UI
            elif macros.get("A") == MKS_SENSOR_PIRANI:
                self.pdispA.filename = PR_UI

            if macros.get("B") == MKS_SENSOR_COLD_CATHODE:
                self.pdispB.filename = CC_UI
            elif macros.get("B") == MKS_SENSOR_PIRANI:
                self.pdispB.filename = PR_UI

            if macros.get("C") == MKS_SENSOR_COLD_CATHODE:
                self.pdispC.filename = CC_UI
            elif macros.get("C") == MKS_SENSOR_PIRANI:
                self.pdispC.filename = PR_UI

        self.pdispA.setAutoFillBackground(False)
        self.pdispB.setAutoFillBackground(False)
        self.pdispC.setAutoFillBackground(False)

    def ui_filename(self):
        return SETTINGS_UI

    def ui_filepath(self):
        return SETTINGS_UI



#!/usr/bin/env python3
import json

from pydm import Display
from qtpy.QtCore import QUrl
from qtpy.QtGui import QDesktopServices

from siriushlacon.mks937b.consts import DEVICE_MENU_UI, INFO_UI, PRESSURE, SETTINGS
from siriushlacon.tools.consts import VIEWER_URL
from siriushlacon.widgets.images import CNPEM_PIXMAP, LNLS_PIXMAP


def get_json_macro(macros):
    return json.dumps(macros)


class DeviceMenu(Display):
    def __init__(self, parent=None, args=None, macros=None):
        super(DeviceMenu, self).__init__(parent=parent, args=args, macros=macros)
        json_macro = get_json_macro(macros)

        self.btnMON.filenames = [PRESSURE]
        self.btnMON.macros = json_macro

        self.btnSettings.filenames = [SETTINGS]
        self.btnSettings.macros = json_macro

        self.btnInfo.filenames = [INFO_UI]
        self.btnInfo.macros = json_macro

        self.label_cnpem.setPixmap(CNPEM_PIXMAP)
        self.label_lnls.setPixmap(LNLS_PIXMAP)

    def open_archiver(self):
        QDesktopServices.openUrl(QUrl(VIEWER_URL))

    def ui_filename(self):
        return DEVICE_MENU_UI

    def ui_filepath(self):
        return DEVICE_MENU_UI
